@@ -1,4 +1,22 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
+#
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+## TODO: Remove/replace this program
+
 # simple announcer terminal for old metarace track meet
 #
 # approx 80x28, addressed with UNT4/RTF style line/pos/erp/erl
@@ -341,7 +359,7 @@ def main():
     app.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
         app.shutdown()
         raise
