@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,7 +19,10 @@
 
 This module provides a class 'classification' which implements the 'race'
 interface and manages data, timing and scoreboard for an overall
-classification.
+classification. Classification can also be used to aggregate a multi-part
+event from other primitive elements, and then report them grouped in
+result and export results.
+
 
 Notes:
 
@@ -100,14 +105,15 @@ class classification(object):
                            + repr(eid) + ' != ' + repr(EVENT_ID))
             self.readonly = True
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
         sec = printing.section()
-        sec.heading = u' '.join([
-                      u'Event', self.evno, u':', self.event[u'pref'],
-                         self.event[u'info'],
-                      u'- Start List'])
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
