@@ -1,4 +1,6 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +23,7 @@ UCI flying lap time trial.
 
 Peculiarities when compared to ittt:
 
- - times are truncated to thousandths
+ - times are hard truncated to thousandths
  - dead heat on thousandths is broken by time over last 200m
  - autoarm is the default
 
