@@ -208,8 +208,10 @@ class hl970:
                             dstr = u'+' + (self.tod - self.elapfin).rawtime(0)
                 else:
                     # race in progress, show run time and distance or lap
-                    estr = (self.tod -
-                            (self.elapstart+tod.tod('0.5'))).rawtime(0)
+                    elap = self.tod - (self.elapstart+tod.tod('0.2'))
+                    if elap > tod.MAXELAP:
+                        elap = tod.ZERO
+                    estr = elap.rawtime(0)
                     if self.distance is not None and self.distance > 0.5:
                         dstr = u'~{0:1.1f}km'.format(self.distance)
                     if self.lapfin is not None:
