@@ -1,4 +1,7 @@
 
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -130,7 +133,7 @@ class irtt(object):
                     self.armfinish()
                     return True
                 elif key == key_startlist:
-                    self.meet.scratch_clear()
+                    self.meet.announce_clear()
                     glib.idle_add(self.delayed_announce)
                     return True
                 elif key == key_raceover:
@@ -194,8 +197,11 @@ class irtt(object):
 
     def delayed_announce(self):
         """Re-announce all riders from the nominated category."""
-        self.meet.scb.clrall()
-        self.meet.scb.set_title(self.title_namestr.get_text())
+        self.meet.announce_clear()
+        #self.meet.scb.clrall()
+        self.meet.announce_title(self.title_namestr.get_text())
+        #self.meet.scb.set_title(self.title_namestr.get_text())
+        self.meet.announce_cmd(u'finstr', self.meet.get_short_name())
         cat = self.ridercat(self.curcat)
         for t in self.results[cat]:
             r = self.getiter(t.refid, t.index)
@@ -205,7 +211,8 @@ class irtt(object):
                 rank = self.riders.get_value(r, COL_PLACE)
                 cat = self.riders.get_value(r, COL_CAT)
                 namestr = self.riders.get_value(r, COL_NAMESTR)
-                self.meet.scb.add_rider([rank,bib,namestr,cat,et.rawtime(2)])
+                self.meet.announce_rider([rank,bib,namestr,cat,et.rawtime(2)])
+                #self.meet.scb.add_rider([rank,bib,namestr,cat,et.rawtime(2)])
       
         return False
 
@@ -736,40 +743,71 @@ class irtt(object):
             self.riders.reorder([a[0] for a in aux])
         return cnt
 
-## CHECK!!
 #hh:mm:ss ______ -no rider------ cat
     def startlist_report(self):
+        """Return a startlist report."""
+        self.reorder_startlist()
+        ret = []
+        if len(self.cats) > 1:
+            for c in self.cats:
+                if c:
+                    ret.extend(self.startlist_report_gen(c))
+                    ret.append(printing.pagebreak())
+        else:
+            ret = self.startlist_report_gen()
+        return ret
+
+    def startlist_report_gen(self, cat=None):
+        catname = u''
+        subhead = u''
+        if cat is not None:
+            dbr = self.meet.rdb.getrider(cat,u'cat')
+            if dbr is not None:
+                catname = self.meet.rdb.getvalue(dbr,
+                                          riderdb.COL_FIRST)
+                subhead = self.meet.rdb.getvalue(dbr,
+                                          riderdb.COL_LAST)
+        else:
+            cat = u''   # match all riders
+
         """Return a startlist report (rough style)."""
         ret = []
         sec = printing.rttstartlist()
         sec.heading = 'Startlist'
-        cnt = self.reorder_startlist()
+        if catname:
+            sec.heading += u': ' + catname
+            sec.subheading = subhead
+        rcnt = 0
+        cat = self.ridercat(cat)
         lt = None
         for r in self.riders:
-            ucicode = None
-            dbr = self.meet.rdb.getrider(r[COL_BIB],r[COL_SERIES])
-            if dbr is not None:
-                ucicode = self.meet.rdb.getvalue(dbr,
-                                         riderdb.COL_UCICODE)
-            bstr = r[COL_BIB].decode('utf-8')
-            stxt = ''
-            if r[COL_WALLSTART] is not None:
-                stxt = r[COL_WALLSTART].meridian()
-                if lt is not None: 
-                    if r[COL_WALLSTART] - lt > tod.tod('1:00'):
-                        sec.lines.append([None, None, None])	# GAP!!
-                lt = r[COL_WALLSTART]
-            nstr = r[COL_NAMESTR]
-            #if r[COL_CAT] == 'U23':
-                #ucicode += ' *'
-            cat = None
-            if r[COL_CAT]:
-                cat = r[COL_CAT].decode('utf-8')
-            sec.lines.append([stxt, bstr, nstr, ucicode, '____', cat])
+            rcat = r[COL_CAT].decode('utf-8').upper()
+            if cat == u'' or rcat == cat:
+                rcnt += 1
+                ucicode = None
+                dbr = self.meet.rdb.getrider(r[COL_BIB],r[COL_SERIES])
+                if dbr is not None:
+                    ucicode = self.meet.rdb.getvalue(dbr,
+                                             riderdb.COL_UCICODE)
+                bstr = r[COL_BIB].decode('utf-8')
+                stxt = ''
+                if r[COL_WALLSTART] is not None:
+                    stxt = r[COL_WALLSTART].meridian()
+                    if lt is not None: 
+                        if r[COL_WALLSTART] - lt > self.startgap:
+                            sec.lines.append([None, None, None])	# GAP!!
+                    lt = r[COL_WALLSTART]
+                nstr = r[COL_NAMESTR]
+                #if r[COL_CAT] == 'U23':
+                    #ucicode += ' *'
+                cstr = None
+                if r[COL_CAT]:	# Use cat abbrev in road TT
+                    cstr = r[COL_CAT].decode('utf-8')
+                sec.lines.append([stxt, bstr, nstr, ucicode, '____', cstr])
         ret.append(sec)
-        if cnt > 1:
-            sec = printing.section()
-            sec.lines.append([None, None, 'Total riders: ' + str(cnt)])
+        if rcnt > 1:
+            sec = printing.bullet_text()
+            sec.lines.append([u'', 'Total riders: ' + unicode(rcnt)])
             ret.append(sec)
         return ret
 
@@ -887,10 +925,144 @@ class irtt(object):
 
     def catresult_report(self):
         """Return a categorised result report."""
-        return None
+        self.placexfer()	# ensure all cat places are filled
+        ret = []
+        for cat in self.cats:
+            if not cat:
+                continue        # ignore empty and None cat
+            ret.extend(self.single_catresult(cat))
+        return ret
+
+    def single_catresult(self, cat):
+        ret = []
+        catname = cat   # fallback emergency, cat is never '' here
+        subhead = u''
+        distance = None
+        dbr = self.meet.rdb.getrider(cat,u'cat')
+        if dbr is not None:
+            catname = self.meet.rdb.getvalue(dbr,
+                                      riderdb.COL_FIRST) # already decode
+            subhead = self.meet.rdb.getvalue(dbr,
+                                      riderdb.COL_LAST)
+            dist = self.meet.rdb.getvalue(dbr,
+                                      riderdb.COL_REFID)
+            try:
+                distance = float(dist)
+            except:
+                self.log.warn(u'Invalid distance: ' + repr(dist)
+                               + u' for cat ' + repr(cat))
+        sec = printing.section()
+        rsec = sec
+        ret.append(sec)
+        ct = None
+        lt = None
+        lpstr = None
+        totcount = 0
+        dnscount = 0
+        dnfcount = 0
+        hdcount = 0
+        fincount = 0
+        for r in self.riders:	# scan whole list even though cat are sorted.
+            if cat == self.ridercat(r[COL_CAT].decode('utf-8')):
+                placed = False
+                totcount += 1
+                i = self.getiter(r[COL_BIB], r[COL_SERIES])
+                ft = self.getelapsed(i)
+                bstr = r[COL_BIB].decode('utf-8')
+                nstr = r[COL_NAMESTR].decode('utf-8')
+                cstr = u''
+                if cat == u'':	# categorised result does not need cat
+                    cstr = r[COL_CAT].decode('utf-8')
+                ucicode = None
+                dbr = self.meet.rdb.getrider(r[COL_BIB],r[COL_SERIES])
+                if dbr is not None:
+                    ucicode = self.meet.rdb.getvalue(dbr,
+                                             riderdb.COL_UCICODE)
+                if ucicode:
+                    # overwrite category string
+                    cstr = ucicode
+                if ct is None:
+                    ct = ft
+                pstr = None
+                if r[COL_PLACE] != '' and r[COL_PLACE].isdigit():
+                    pstr = (r[COL_PLACE] + '.')
+                    fincount += 1	# only count placed finishers
+                    placed = True
+                else:
+                    pstr = r[COL_COMMENT]
+                    # 'special' dnfs
+                    if pstr == u'dns':
+                        dnscount += 1
+                    elif pstr == u'hd':
+                        hdcount += 1
+                    else:
+                        if pstr:	# commented dnf
+                            dnfcount += 1
+                    if pstr:
+                        placed = True
+                        if lpstr != pstr:
+                            ## append an empty row
+                            sec.lines.append([None, None, None,
+                                              None, None, None])
+                            lpstr = pstr
+                tstr = None
+                if ft is not None:
+                    tstr = ft.rawtime(2)
+                dstr = None
+                if ct is not None and ft is not None and ct != ft:
+                    dstr = dstr = ('+' + (ft - ct).rawtime(1))
+                if placed:
+                    sec.lines.append([pstr, bstr, nstr, cstr, tstr, dstr])
+
+        residual = totcount - (fincount + dnfcount + dnscount + hdcount)
+
+#!!! relocate to after uci lines
+        if self.timerstat == 'finished':	# THIS OVERRIDES RESIDUAL
+            sec.heading = u'Result'
+        else:
+            if residual > 0 :
+                sec.heading = u'Virtual Standings'
+            else:
+                sec.heading = u'Provisional Result'
+
+        # Race metadata / UCI comments
+        sec = printing.bullet_text()
+        if ct is not None:
+            if distance is not None:
+                avgprompt = u'Average speed of the winner: '
+                if residual > 0:
+                    avgprompt = u'Average speed of the leader: '
+                sec.lines.append([None, avgprompt
+                                    + ct.speedstr(1000.0*distance)])
+        sec.lines.append([None,
+                          u'Number of starters: '
+                          + unicode(totcount-dnscount)])
+        if hdcount > 0:
+            sec.lines.append([None,
+                          u'Riders finishing out of time limits: '
+                          + unicode(hdcount)])
+        if dnfcount > 0:
+            sec.lines.append([None,
+                          u'Riders abandoning the race: '
+                          + unicode(dnfcount)])
+        ret.append(sec)
+
+        # finish report title manipulation
+        if catname:
+            rsec.heading += u': ' + catname
+            rsec.subheading = subhead
+        ret.append(printing.pagebreak())
+        return ret
+
 
     def result_report(self):
         """Return a race result report."""
+
+        # check if a categorised report is required
+        if len(self.cats) > 1:
+            return self.catresult_report()
+
+        # the below is incorrect - remove is a todo
         ret = []
         self.placexfer()
         if self.onestart:
@@ -1188,7 +1360,7 @@ class irtt(object):
 
             # after manips, then re-set start time
             self.sl.set_time(nowoft.timestr(0))
-            self.meet.scb.set_time(nowoft.meridian())
+            #self.meet.scb.set_time(nowoft.meridian())
 
             # if finish lane loaded, set the elapsed time
             if self.fl.getstatus() in ['load', 'running', 'armfin']:
@@ -1626,7 +1798,7 @@ class irtt(object):
 
     def ridercat(self, cat):
         """Return a category from the result for the riders cat."""
-        ret = ''	# default is the 'None' category - uncategorised
+        ret = u''	# default is the 'None' category - uncategorised
         checka = cat.upper()
         if checka in self.results:
             ret = checka
