@@ -1,21 +1,19 @@
 
-# preliminary road race announce 'stateful' terminal
-# connects to IRC to receive UNT4 announce msg packs
+# Metarace : Cycle Race Abstractions
+# Copyright (C) 2012  Nathan Fraser
 #
-# Notes:
-#	- does not _yet_ do live stream negotiation
-#	- server, port, channel, nick config via road_announce.ini
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
 #
-# TODO:
-#	- properly mark private vars/methods to avoid leak to other thread
-#	- configuration dialog and test facility -> query uSCBsrv
-#	- configuration status icon for connect & status
-#	- configure map colours via ini
-#       - connect buttons to allow navigation of 'laps'
-#       - touch screen scroll/zoom in map area
-#       - touch screen scroll in tree
-#       - more natural rider search
-#	- connect riders and scale via mark on drawing area?
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import pygtk
 pygtk.require("2.0")
@@ -595,7 +593,7 @@ def main():
     app.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
         app.shutdown()
         raise
