@@ -66,8 +66,13 @@ COL_SPLITS = 9
 # scb consts
 SCB_NAME_WIDTH = metarace.SCB_LINELEN - 9
 SCB_RESNAME_WIDTH = metarace.SCB_LINELEN - 11
-SCB_RESULT_FMT = [(2, u'l'), (3, u'r'), u' ',
-                  (SCB_RESNAME_WIDTH, u'l'), u' ', (4, u'r')]
+SCB_RESULT_FMT = [(3, u'l'), (3, u'r'), u' ',
+                  (SCB_RESNAME_WIDTH, u'l'), u' ', (3, u'r')]
+
+# team name specialisations
+SCB_TEAMNAME_WIDTH = metarace.SCB_LINELEN
+SCB_RESTEAM_WIDTH = metarace.SCB_LINELEN - 4
+SCB_TEAM_FMT = [(4, u'l'), (SCB_RESTEAM_WIDTH,u'l')]
 
 # scb function key mappings
 key_reannounce = 'F4'		     # (+CTRL) calls into delayed announce
@@ -157,16 +162,28 @@ class ittt(object):
         fmtplaces = []
         for r in self.riders:
             if r[COL_PLACE] is not None and r[COL_PLACE] != '':
-                if r[COL_PLACE].isdigit() and int(r[COL_PLACE]) > 20:
-                    break
-                fmtplaces.append([r[COL_PLACE].decode('utf-8'),
-                          r[COL_BIB].decode('utf-8'),
-                          strops.fitname(r[COL_FIRSTNAME].decode('utf-8'),
-                          r[COL_LASTNAME].decode('utf-8'), SCB_RESNAME_WIDTH),
-                          r[COL_CLUB].decode('utf-8')])
+                plstr = r[COL_PLACE].decode('utf-8')
+                if plstr.isdigit():
+                    if int(plstr) > 20:
+                        break
+                    plstr = plstr + u'.'
+                if not self.teamnames:
+                    fmtplaces.append([plstr,
+                         r[COL_BIB].decode('utf-8'),
+                         strops.fitname(r[COL_FIRSTNAME].decode('utf-8'),
+                         r[COL_LASTNAME].decode('utf-8'), SCB_RESNAME_WIDTH),
+                         r[COL_CLUB].decode('utf-8')])
+                else:
+                    fmtplaces.append([plstr,
+                         strops.truncpad(r[COL_FIRSTNAME].decode('utf-8'),
+                                         SCB_RESTEAM_WIDTH, 'l')])
+        evtstatus = u'STANDINGS'	# TODO: detect result/virtual/prov
+        fmt = SCB_RESULT_FMT
+        if self.teamnames:
+            fmt = SCB_TEAM_FMT
         self.meet.scbwin = scbwin.scbtable(self.meet.scb,
-                       self.meet.racenamecat(self.event),
-                       SCB_RESULT_FMT, fmtplaces, delay=80)
+                       self.meet.racenamecat(self.event, tail=evtstatus),
+                       fmt, fmtplaces, delay=50)
         self.meet.scbwin.reset()
 
     def todstr(self, col, cr, model, iter, data=None):
@@ -295,6 +312,8 @@ class ittt(object):
                     nr[i] = self.meet.rdb.getvalue(dbr, i)
             nri = self.riders.append(nr)
             self.settimes(nri, st, ft, sp, doplaces=False, comment=co)
+        if self.series and self.series[0].lower() == u't':
+            self.teamnames = True
         self.placexfer()
 
         # re-join any existing timer state
@@ -388,7 +407,7 @@ class ittt(object):
         with open(self.configpath, 'wb') as f:
             cw.write(f)
 
-    def startlist_report(self):
+    def startlist_report(self, program=False):
         """Return a startlist report."""
         ret = []
         cnt = 0
@@ -397,10 +416,12 @@ class ittt(object):
         if self.timetype == 'single':
             sec.set_single()
 
-        sec.heading = u' '.join([
-                      u'Event', self.evno, u':',
-                      self.event[u'pref'], self.event[u'info'],
-                      u'- Start List'])
+        headvec = [u'Event', self.evno, u':', self.event[u'pref'],
+                         self.event[u'info']]
+        if not program:
+            headvec.append(u'- Start List')
+        sec.heading = u' '.join(headvec)
+
         lapstring = strops.lapstring(self.event[u'laps'])
         substr = u' '.join([lapstring, self.event[u'dist'],
                              self.event[u'prog']]).strip()
@@ -606,8 +627,13 @@ class ittt(object):
                 if r[COL_CLUB] != u'' and len(r[COL_CLUB]) <= 3:
                     clubstr = u' (' + r[COL_CLUB] + u')'
  
-                namestr = strops.truncpad(strops.fitname(r[COL_FIRSTNAME],
+                namestr = u''
+                if not self.teamnames:
+                    namestr = strops.truncpad(strops.fitname(r[COL_FIRSTNAME],
                               r[COL_LASTNAME], 20-len(clubstr))+clubstr, 20)
+                else:
+                    namestr = strops.truncpad(r[COL_FIRSTNAME].decode('utf-8'),
+                                              20)
                 placestr = u'   ' # 3 ch
                 if r[COL_PLACE] != u'':
                     placestr = strops.truncpad(r[COL_PLACE] + u'.', 3)
@@ -747,11 +773,15 @@ class ittt(object):
         ftime = None
         for r in self.riders:
             rno = r[COL_BIB]
-            if u't' in self.series:	# Team no hack
-                rno = u' '	# force name
             rank = None
-            rname = strops.resname(r[COL_FIRSTNAME], r[COL_LASTNAME],
-                                      r[COL_CLUB])
+            rname = u''
+            if not self.teamnames:
+                rname = strops.resname(r[COL_FIRSTNAME].decode('utf-8'),
+                                       r[COL_LASTNAME].decode('utf-8'),
+                                       r[COL_CLUB].decode('utf-8'))
+            else:	# Team no hack
+                rno = u' '	# force name
+                rname = r[COL_FIRSTNAME].decode('utf-8')
             rtime = None
             rcat = None
             if self.showcats:
@@ -1195,17 +1225,24 @@ class ittt(object):
         if bib != '':
             name = '[New Rider]'
             r = self.getrider(bib)
+            ret = name
             if r is not None and r[COL_BIB] != '':
-                first = r[COL_FIRSTNAME].decode('utf-8')
-                last = r[COL_LASTNAME].decode('utf-8')
-                club = r[COL_CLUB].decode('utf-8')
-                name = strops.fitname(first,
-                                      last,
-                                      SCB_NAME_WIDTH)
-                tp.namevec = [bib, strops.resname(first, last, club), '']
-            return ' '.join([strops.truncpad(r[COL_BIB], 3, 'r'),
-                             strops.truncpad(name, SCB_NAME_WIDTH),
-                             strops.truncpad(club, 3, 'r')])
+                if self.teamnames:
+                    ret = strops.truncpad(r[COL_FIRSTNAME].decode('utf-8'),
+                                           SCB_TEAMNAME_WIDTH, 'l')
+                    tp.namevec = [bib, ret, '']
+                else:
+                    first = r[COL_FIRSTNAME].decode('utf-8')
+                    last = r[COL_LASTNAME].decode('utf-8')
+                    club = r[COL_CLUB].decode('utf-8')
+                    name = strops.fitname(first,
+                                          last,
+                                          SCB_NAME_WIDTH)
+                    tp.namevec = [bib, strops.resname(first, last, club), '']
+                    ret = ' '.join([strops.truncpad(r[COL_BIB], 3, 'r'),
+                                 strops.truncpad(name, SCB_NAME_WIDTH),
+                                 strops.truncpad(club, 3, 'r')])
+            return ret
         else:
             tp.namevec = None
             return ''
@@ -1217,6 +1254,8 @@ class ittt(object):
                                 self.meet.racenamecat(self.event),
                                 self.fmtridername(self.fs),
                                 self.fmtridername(self.bs))
+        if self.timetype == 'single':
+            self.meet.scbwin.set_single()
         self.meet.gemini.reset_fields()
         self.meet.announce.gfx_overlay(2)
         self.meet.announce.gfx_set_title(self.event[u'pref'] + u' '
@@ -1475,6 +1514,7 @@ class ittt(object):
         self.autoarm = False
         self.difftime = False
         self.teampursuit = False
+        self.teamnames = False		# team names only shown
         self.chan_A = timy.CHAN_PA	# default is ITT/Pursuit
         self.chan_B = timy.CHAN_PB
         self.chan_S = timy.CHAN_START
