@@ -1,12 +1,3 @@
-## TODO:	- merge autoclear announce into notepad
-##		- unified announcer output
-##		- multi-announce framework
-##		- remote control via uscbsrv
-##		- export triggers
-##		- standardise notepad formats for laps, passings, etc
-##		- begin uscbsrv bot with proper announce handshake
-##		- save all RFID data for later analysis, or re-play
-##		- finalise export/import semantics and interfaces
 
 # Metarace : Cycle Race Abstractions
 # Copyright (C) 2012  Nathan Fraser
@@ -49,13 +40,14 @@ from metarace import riderdb
 from metarace import thbc
 from metarace import wheeltime
 from metarace import timy
-from metarace import uscbsrv
+from metarace import telegraph
 from metarace import unt4
 from metarace import strops
 from metarace import loghandler
 from metarace import uiutil
 from metarace import printing
 from metarace import rsync
+from metarace import namebank
 
 LOGHANDLER_LEVEL = logging.DEBUG
 ROADRACE_TYPES = {u'irtt':u'Road Time Trial',
@@ -183,6 +175,7 @@ class registerdlg(object):
                 # rider ok to assign or new rider
                 bib = self.bibent.get_text().decode('utf-8')
                 ser = self.serent.get_text().decode('utf-8')
+                self.bibent.activate()	# required?
                 r = self.rdb.getrider(bib, ser)
                 if r is not None:
                     # check for existing tag allocation
@@ -209,7 +202,7 @@ class fakemeet:
         self.timer = thbc.thbc()
         self.alttimer = timy.timy()
         self.stat_but = gtk.Button()
-        self.scb = uscbsrv.uscbsrv()
+        self.scb = telegraph.telegraph()
         self.title_str = ''
         self.date_str = ''
         self.organiser_str = ''
@@ -255,7 +248,7 @@ class fakemeet:
         pass
     def loadconfig(self):
         """Load meet config from disk."""
-        cr = jsonconfig.config({u'meet':{
+        cr = jsonconfig.config({u'roadmeet':{
                u'title':u'',
                u'shortname':u'',
                u'subtitle':u'',
@@ -269,7 +262,8 @@ class fakemeet:
                u'resultnos':u'Yes',
                u'uscbport':u'',
                u'id':u''}})
-        cr.add_section(u'meet')
+        cr.add_section(u'roadmeet')
+        cr.merge(metarace.sysconf, u'roadmeet')
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         # check for config file
         try:
@@ -278,22 +272,27 @@ class fakemeet:
         except Exception as e:
             self.log.error(u'Reading meet config: ' + unicode(e))
         # set meet meta, and then copy into text entries
-        self.shortname = cr.get(u'meet', u'shortname')
-        self.title_str = cr.get(u'meet', u'title')
-        self.subtitle_str = cr.get(u'meet', u'subtitle')
-        self.document_str = cr.get(u'meet', u'document')
-        self.date_str = cr.get(u'meet', u'date')
-        self.organiser_str = cr.get(u'meet', u'organiser')
-        self.commissaire_str = cr.get(u'meet', u'commissaire')
-        self.logo = cr.get(u'meet', u'logo')
-        self.distance = strops.confopt_float(cr.get(u'meet', u'distance'))
-        self.docindex = strops.confopt_posint(cr.get(u'meet', u'docindex'), 0)
-        self.bibs_in_results = strops.confopt_bool(cr.get(u'meet',
+        self.shortname = cr.get(u'roadmeet', u'shortname')
+        self.title_str = cr.get(u'roadmeet', u'title')
+        self.subtitle_str = cr.get(u'roadmeet', u'subtitle')
+        self.document_str = cr.get(u'roadmeet', u'document')
+        self.date_str = cr.get(u'roadmeet', u'date')
+        self.organiser_str = cr.get(u'roadmeet', u'organiser')
+        self.commissaire_str = cr.get(u'roadmeet', u'commissaire')
+        self.logo = cr.get(u'roadmeet', u'logo')
+        self.distance = strops.confopt_float(cr.get(u'roadmeet', u'distance'))
+        self.docindex = strops.confopt_posint(cr.get(u'roadmeet', u'docindex'), 0)
+        self.bibs_in_results = strops.confopt_bool(cr.get(u'roadmeet',
                                                           u'resultnos'))
     def event_configfile(self, evno):
         """Return a config filename for the given event no."""
         return os.path.join(self.configpath, u'event_' 
                                      + unicode(evno) + u'.json')
+    def default_template(self):
+        tfile = os.path.join(self.configpath, u'template.json')
+        if not os.path.exists(tfile):
+            tfile = None
+        return tfile
 
 class roadmeet:
     """Road meet application class."""
@@ -327,6 +326,10 @@ class roadmeet:
         """Return the <= 16 char shortname."""
         return self.shortname
 
+    def cat_but_auto_clicked(self, but, entry, data=None):
+        """Lookup cats and write them into the supplied entry."""
+        entry.set_text(' '.join(self.rdb.listcats()))
+        
     ## TODO: call into curevent and fetch configurables + use generic props
     def menu_meet_properties_cb(self, menuitem, data=None):
         """Edit meet properties."""
@@ -356,6 +359,14 @@ class roadmeet:
             tcombo.set_sensitive(False)
             tmodel.append([self.etype,tlbl])
 
+        # fetch event result categories
+        cat_ent = b.get_object('cat_entry')
+        if self.curevent is not None:
+            clist = u' '.join(self.curevent.get_catlist())
+            cat_ent.set_text(clist)
+            cba = b.get_object('cat_but_auto')
+            cba.connect('clicked', self.cat_but_auto_clicked, cat_ent)
+
         # fill text entries
         t_ent = b.get_object('title_entry')
         t_ent.set_text(self.title_str)
@@ -383,7 +394,7 @@ class roadmeet:
         uce.set_text(self.uscbchan)
         upb = b.get_object('scb_port_dfl')
         upb.connect('clicked', 
-                    lambda b: upe.set_text('uSCBsrv@localhost'))
+                    lambda b: upe.set_text('dhm@localhost'))
         uoc = b.get_object('uscbsrv_option_check')
         uoc.set_active(self.remote_enable)
         mte = b.get_object('timing_main_entry')
@@ -426,6 +437,9 @@ class roadmeet:
                 self.alttimer_port = nport
                 self.alttimer.setport(nport)
 
+            if self.curevent is not None:
+                self.curevent.loadcats(cat_ent.get_text().decode('utf-8'))
+
             if dotype:
                 # check for type change
                 nt = tmodel.get_value(tcombo.get_active_iter(), 0)
@@ -469,15 +483,20 @@ class roadmeet:
         for sec in sections:
             rep.add_section(sec)
         print_op = gtk.PrintOperation()
+        print_op.set_allow_async(True)
         print_op.set_print_settings(self.printprefs)
         print_op.set_default_page_setup(self.pageset)
         print_op.connect("begin_print", self.begin_print, rep)
         print_op.connect("draw_page", self.draw_print_page, rep)
         res = print_op.run(gtk.PRINT_OPERATION_ACTION_PREVIEW,
-                               self.window)
+                               None)
+        #res = print_op.run(gtk.PRINT_OPERATION_ACTION_PREVIEW,
+                               #self.window)
         if res == gtk.PRINT_OPERATION_RESULT_APPLY:
             self.printprefs = print_op.get_print_settings()
             self.log.debug(u'Updated print preferences.')
+        elif res == gtk.PRINT_OPERATION_RESULT_IN_PROGRESS:
+            self.log.debug(u'Print operation running asynchronously.')
         self.docindex += 1
 
         ofile = os.path.join(self.configpath, u'output.pdf')
@@ -592,6 +611,17 @@ class roadmeet:
             sections = self.curevent.startlist_report()
             if sections:
                 self.print_report(sections)
+            else:
+                self.log.info('Startlist - Nothing to print.')
+
+    def menu_reports_signon_activate_cb(self, menuitem, data=None):
+        """Generate a sign on sheet."""
+        if self.curevent is not None:
+            sections = self.curevent.signon_report()
+            if sections:
+                self.print_report(sections)
+            else:
+                self.log.info('Sign-on - Nothing to print.')
 
     def menu_reports_camera_activate_cb(self, menuitem, data=None):
         """Generate the camera operator report."""
@@ -636,22 +666,18 @@ class roadmeet:
         rdlg.destroy()
 
     def menu_data_uscb_activate_cb(self, menuitem, data=None):
-        """Request a re-connect to the uSCBsrv IRC connection."""
-        if self.uscbport:
-            self.log.info(u'Requesting re-connect to announcer: ' + 
-                           unicode(self.uscbport) + unicode(self.uscbchan))
-        else:
-            self.log.info(u'Announcer not configured.')
-            # but still call into uscbsrv...
-        self.scb.set_portstr(portstr=self.uscbport,
-                             channel=self.uscbchan,
-                             force=True)
+        """Reload rider db from disk."""
+        self.rdb.clear()
+        self.rdb.load(os.path.join(self.configpath, u'riders.csv'))
+        self.log.info(u'Reloaded riders from disk.')
+        self.menu_race_run_activate_cb()
 
     def menu_import_riders_activate_cb(self, menuitem, data=None):
         """Add riders to database."""
         sfile = uiutil.loadcsvdlg(u'Select rider file to import', self.window)
         if sfile is not None:
-            self.rdb.load(sfile)
+            with namebank.namebank() as n:            
+                self.rdb.load(sfile, namedb=n, overwrite=True)
         else:
             self.log.debug(u'Import riders cancelled.')
 
@@ -812,7 +838,15 @@ class roadmeet:
         # 4: call into curevent to get result sections
         ## TODO: if status is 'new' write out a startlist
         ##       else result, but use catresult if flag in config
-        if self.curevent.timerstat in ['idle', 'ready']:
+        dostart = False	# assume not exporting startlist
+        if self.etype == u'irtt':
+            if not self.curevent.onestart:
+                dostart = True
+        else:
+            if self.curevent.timerstat in ['idle', 'ready']:
+                dostart = True
+
+        if dostart:	# emit startlist(s) instead of result
             for sec in self.curevent.startlist_report():
                 rep.add_section(sec)
         else:
@@ -820,7 +854,10 @@ class roadmeet:
                 rep.add_section(sec)
 
         # 5: write out files
-        filebase = os.path.basename(self.configpath)
+        if self.mirrorfile:
+            filebase = self.mirrorfile
+        else:
+            filebase = os.path.basename(self.configpath)
         if filebase in [u'', u'.']:
             filebase = u'result'
             self.log.warn(u'Using default filename for export: result')
@@ -834,8 +871,11 @@ class roadmeet:
         ofile = os.path.join(exportpath, filebase+u'.xls')
         with open(ofile, 'wb') as f:
             rep.output_xls(f)
+        ofile = os.path.join(exportpath, filebase+u'.json')
+        with open(ofile, 'wb') as f:
+            rep.output_json(f)
         lb = os.path.join(u'/site', self.mirrorpath, filebase)
-        lt = [u'pdf', u'xls']
+        lt = [u'pdf', u'xls', u'txt', u'json']
         ofile = os.path.join(exportpath, filebase+u'.txt')
         with open(ofile, 'wb') as f:
             rep.output_text(f, linkbase=lb, linktypes=lt)
@@ -981,7 +1021,7 @@ class roadmeet:
             typepfx = ROADRACE_TYPES[self.etype] + u' :: '
         self.window.set_title(typepfx + title)
         if self.curevent is not None:
-            self.curevent.set_titlestr(title)
+            self.curevent.set_titlestr(self.subtitle_str)
 
     def meet_destroy_cb(self, window, msg=u''):
         """Handle destroy signal and exit application."""
@@ -1058,30 +1098,31 @@ class roadmeet:
         if self.curevent is not None and self.curevent.winopen:
             self.curevent.saveconfig()
         cw = jsonconfig.config()
-        cw.add_section(u'meet')
-        cw.set(u'meet', u'id', ROADMEET_ID)
-        cw.set(u'meet', u'uscbport', self.uscbport)
-        cw.set(u'meet', u'uscbchan', self.uscbchan)
-        cw.set(u'meet', u'uscbopt', self.remote_enable)
-        cw.set(u'meet', u'timer', self.timer_port)
-        cw.set(u'meet', u'rfidlvl', self.rfidlvl)
-        cw.set(u'meet', u'alttimer', self.alttimer_port)
+        cw.add_section(u'roadmeet')
+        cw.set(u'roadmeet', u'id', ROADMEET_ID)
+        cw.set(u'roadmeet', u'uscbport', self.uscbport)
+        cw.set(u'roadmeet', u'uscbchan', self.uscbchan)
+        cw.set(u'roadmeet', u'uscbopt', self.remote_enable)
+        cw.set(u'roadmeet', u'timer', self.timer_port)
+        cw.set(u'roadmeet', u'rfidlvl', self.rfidlvl)
+        cw.set(u'roadmeet', u'alttimer', self.alttimer_port)
 
-        cw.set(u'meet', u'shortname', self.shortname)
-        cw.set(u'meet', u'title', self.title_str)
-        cw.set(u'meet', u'subtitle', self.subtitle_str)
-        cw.set(u'meet', u'document', self.document_str)
-        cw.set(u'meet', u'date', self.date_str)
-        cw.set(u'meet', u'organiser', self.organiser_str)
-        cw.set(u'meet', u'commissaire', self.commissaire_str)
-        cw.set(u'meet', u'logo', self.logo)
+        cw.set(u'roadmeet', u'shortname', self.shortname)
+        cw.set(u'roadmeet', u'title', self.title_str)
+        cw.set(u'roadmeet', u'subtitle', self.subtitle_str)
+        cw.set(u'roadmeet', u'document', self.document_str)
+        cw.set(u'roadmeet', u'date', self.date_str)
+        cw.set(u'roadmeet', u'organiser', self.organiser_str)
+        cw.set(u'roadmeet', u'commissaire', self.commissaire_str)
+        cw.set(u'roadmeet', u'logo', self.logo)
 
-        cw.set(u'meet', u'resultnos', self.bibs_in_results)
-        cw.set(u'meet', u'distance', self.distance)
-        cw.set(u'meet', u'docindex', self.docindex)
-        cw.set(u'meet', u'loglevel', self.loglevel)
-        cw.set(u'meet', u'mirrorpath', self.mirrorpath)
-        cw.set(u'meet', u'mirrorcmd', self.mirrorcmd)
+        cw.set(u'roadmeet', u'resultnos', self.bibs_in_results)
+        cw.set(u'roadmeet', u'distance', self.distance)
+        cw.set(u'roadmeet', u'docindex', self.docindex)
+        cw.set(u'roadmeet', u'loglevel', self.loglevel)
+        cw.set(u'roadmeet', u'mirrorpath', self.mirrorpath)
+        cw.set(u'roadmeet', u'mirrorcmd', self.mirrorcmd)
+        cw.set(u'roadmeet', u'mirrorfile', self.mirrorfile)
 
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         self.log.debug(u'Saving meet config to ' + repr(cwfilename))
@@ -1116,7 +1157,7 @@ class roadmeet:
 
     def loadconfig(self):
         """Load meet config from disk."""
-        cr = jsonconfig.config({u'meet':{
+        cr = jsonconfig.config({u'roadmeet':{
                u'shortname':None,
                u'title':u'',
                u'subtitle':u'',
@@ -1132,13 +1173,15 @@ class roadmeet:
                u'alttimer':u'',
                u'resultnos':u'Yes',
                u'uscbport':u'',
-               u'uscbchan':u'#announce',
+               u'uscbchan':u'',
                u'uscbopt':u'No',
                u'mirrorpath':u'',
                u'mirrorcmd':u'echo',
+               u'mirrorfile':u'',
                u'loglevel':unicode(logging.INFO),
                u'id':u''}})
-        cr.add_section(u'meet')
+        cr.add_section(u'roadmeet')
+        cr.merge(metarace.sysconf, u'roadmeet')
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
 
         # re-set log file
@@ -1161,25 +1204,25 @@ class roadmeet:
             self.log.error(u'Reading meet config: ' + unicode(e))
 
         # configurable log level on UI (does not change log file)
-        self.loglevel = strops.confopt_posint(cr.get(u'meet', u'loglevel'),
+        self.loglevel = strops.confopt_posint(cr.get(u'roadmeet', u'loglevel'),
                                               logging.INFO)
         self.lh.setLevel(self.loglevel)
         self.log.debug(u'UI Log level: ' + repr(self.loglevel))
         self.log.debug(u'Reading event config.')
 
         # set uSCBsrv connection
-        self.uscbchan = cr.get(u'meet', u'uscbchan')
-        self.uscbport = cr.get(u'meet', u'uscbport')
+        self.uscbchan = cr.get(u'roadmeet', u'uscbchan')
+        self.uscbport = cr.get(u'roadmeet', u'uscbport')
         self.scb.set_portstr(portstr=self.uscbport,
                              channel=self.uscbchan)
-        self.remote_enable = strops.confopt_bool(cr.get(u'meet', u'uscbopt'))
+        self.remote_enable = strops.confopt_bool(cr.get(u'roadmeet', u'uscbopt'))
         self.remote_reset()
 
         # set timer port (decoder)
-        self.set_timer(cr.get(u'meet', u'timer'))
+        self.set_timer(cr.get(u'roadmeet', u'timer'))
         boxlvl = u'20'
         stalvl = u'30'
-        nv = cr.get(u'meet', u'rfidlvl')
+        nv = cr.get(u'roadmeet', u'rfidlvl')
         if len(nv) > 1:
             boxlvl = nv[0]
             stalvl = nv[1]
@@ -1187,27 +1230,28 @@ class roadmeet:
         self.rfidlvl = [boxlvl, stalvl]
 
         # set alt timer port (timy)
-        nport = cr.get(u'meet', u'alttimer')
+        nport = cr.get(u'roadmeet', u'alttimer')
         if nport != self.alttimer_port:
             self.alttimer_port = nport
             self.alttimer.setport(nport)
             self.alttimer.sane() # sane prod here is probably good idea
 
         # set meet meta, and then copy into text entries
-        self.shortname = cr.get(u'meet', u'shortname')
-        self.title_str = cr.get(u'meet', u'title')
-        self.subtitle_str = cr.get(u'meet', u'subtitle')
-        self.document_str = cr.get(u'meet', u'document')
-        self.date_str = cr.get(u'meet', u'date')
-        self.organiser_str = cr.get(u'meet', u'organiser')
-        self.commissaire_str = cr.get(u'meet', u'commissaire')
-        self.logo = cr.get(u'meet', u'logo')
-        self.distance = strops.confopt_float(cr.get(u'meet', u'distance'))
-        self.docindex = strops.confopt_posint(cr.get(u'meet', u'docindex'), 0)
-        self.bibs_in_results = strops.confopt_bool(cr.get(u'meet',
+        self.shortname = cr.get(u'roadmeet', u'shortname')
+        self.title_str = cr.get(u'roadmeet', u'title')
+        self.subtitle_str = cr.get(u'roadmeet', u'subtitle')
+        self.document_str = cr.get(u'roadmeet', u'document')
+        self.date_str = cr.get(u'roadmeet', u'date')
+        self.organiser_str = cr.get(u'roadmeet', u'organiser')
+        self.commissaire_str = cr.get(u'roadmeet', u'commissaire')
+        self.logo = cr.get(u'roadmeet', u'logo')
+        self.distance = strops.confopt_float(cr.get(u'roadmeet', u'distance'))
+        self.docindex = strops.confopt_posint(cr.get(u'roadmeet', u'docindex'), 0)
+        self.bibs_in_results = strops.confopt_bool(cr.get(u'roadmeet',
                                                           u'resultnos'))
-        self.mirrorpath = cr.get(u'meet', u'mirrorpath')
-        self.mirrorcmd = cr.get(u'meet', u'mirrorcmd')
+        self.mirrorpath = cr.get(u'roadmeet', u'mirrorpath')
+        self.mirrorcmd = cr.get(u'roadmeet', u'mirrorcmd')
+        self.mirrorfile = cr.get(u'roadmeet', u'mirrorfile')
 
         # Re-Initialise rider and event databases
         self.rdb.clear()
@@ -1237,7 +1281,7 @@ class roadmeet:
         # an error message is left on top of status stack. This is not
         # always a hard fail and the user should be left to determine
         # an appropriate outcome.
-        cid = cr.get(u'meet', u'id')
+        cid = cr.get(u'roadmeet', u'id')
         if cid and cid != ROADMEET_ID:
             self.log.error(u'Meet configuration mismatch: '
                            + repr(cid) + u' != ' + repr(ROADMEET_ID))
@@ -1252,7 +1296,8 @@ class roadmeet:
         if not self.remote_enable:
             tvec = [unicode(1), self.timer.unitno, evt.chan,
                     evt.refid, evt.rawtime()]
-            self.scb.send_cmd(u'timer',unichr(unt4.US).join(tvec))
+            self.scb.send_cmd(u'timer',unichr(unt4.US).join(tvec),
+                                       self.timerchan)
 
     def announce_clear(self):
         """Clear announce panels."""
@@ -1349,13 +1394,9 @@ class roadmeet:
             self.log.debug(u'Disabled.')
             self.scb.set_pub_cb()
 
-    def remote_pub_cb(self, cmd, nick):
+    def remote_pub_cb(self, cmd, nick, chan):
         """Handle a remote control message."""
-        # TODO : are we listening for this nick?
-        # TODO : is the sender authenticated to control?
-        # TODO : is this a message already dealt with?
-        # TODO : is it possible some data was missed?
-        if cmd.header == u'timer':
+        if chan == self.timerchan and cmd.header == u'timer':
             tv = cmd.text.split(unichr(unt4.US)) # split into fields
             if len(tv) > 4:	# enough fields
                 try:
@@ -1415,6 +1456,7 @@ class roadmeet:
         self.pageset.set_right_margin(0, gtk.UNIT_POINTS)
 
         # hardware connections
+        self.timerchan = '#timing'	# raw timing channel on telegraph
         self.timer = thbc.thbc()	# default is Tag, may change later
         self.timer_port = u''
         self.rfidlvl = [u'20',u'30']
@@ -1422,9 +1464,11 @@ class roadmeet:
         self.alttimer_port = u''
         self.uscbport = u''
         self.uscbchan = u'#announce'
-        self.scb = uscbsrv.uscbsrv()
+        self.scb = telegraph.telegraph()
+        self.scb.add_channel(self.timerchan)	# persists over server recon
         self.mirrorpath = u''
         self.mirrorcmd = u'echo'
+        self.mirrorfile = u''
         self.mirror = None 
 
         b = gtk.Builder()
@@ -1447,7 +1491,8 @@ class roadmeet:
         self.action_model = b.get_object('race_action_model')
         self.action_combo = b.get_object('race_action_combo')
         self.action_entry = b.get_object('race_action_entry')
-        b.get_object('race_tab_img').set_from_file(metarace.SCB_LOGOFILE)
+        b.get_object('race_tab_img').set_from_file(
+                                   metarace.default_file(metarace.LOGO_FILE))
         b.get_object('race_stat_hbox').set_focus_chain([self.action_combo,
                                              self.action_entry,
                                              self.action_combo])
@@ -1505,7 +1550,9 @@ class roadmeet:
         # get rider db and pack into a dialog
         self.rdb = riderdb.riderdb()
         b.get_object('riders_box').add(self.rdb.mkview(cat=True,
-                                                  series=False,refid=True))
+                                                  series=True,
+                                                  refid=True,
+                                                  ucicode=True))
 
         # select event page in notebook.
         b.get_object('meet_nb').set_current_page(1)
@@ -1538,9 +1585,9 @@ def main(etype=None):
     app.window.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
-        app.shutdown(u'Exception from gtk.main()')
+        app.shutdown(u'Exception from main loop.')
         raise
 
 if __name__ == '__main__':
