@@ -235,15 +235,16 @@ class rr_announce(object):
         return True
 
     def clear(self):
+        self.title_str = None
         self.laptype = None
         self.lapstart = None
         self.lapfin = None
-        self.lbl_header.set_text(self.motd)
+        #self.lbl_header.set_text(self.motd)
         self.lbl_resultmsg.set_text(u'')
         self.lbl_resultmsg.hide()
         self.final = False
-        self.elap_lbl.set_text('')
-        self.gap_lbl.set_text('')
+        #self.elap_lbl.set_text('')
+        #self.gap_lbl.set_text('')
         self.riders.clear()
         self.redraw_flag = True
         
@@ -313,12 +314,16 @@ class rr_announce(object):
             return False
         if m.header == 'rider':
             self.append_rider(m.text)
-        #elif m.header == 'time':
-            #self.elap_lbl.set_text(m.text)
-        #elif m.header == 'gap':
-            #self.gap_lbl.set_text(m.text)
         elif m.header == 'title':
-            self.lbl_header.set_text(m.text)
+            if m.text:
+                self.title_str = m.text
+            else:
+                self.title_str = None
+        elif m.header == 'finstr':
+            if m.text:
+                self.finstr = m.text
+            else:
+                self.finstr = None
         elif m.header == 'start':
             self.elapstart = tod.str2tod(m.text)
         elif m.header == 'finish':
@@ -350,8 +355,7 @@ class rr_announce(object):
                 if a > 0.1:
                     self.distance = a
             except:
-                #self.log.debug(u'Invalid distance: ' + repr(cmd.text))
-                print(u'Invalid distance: ' + repr(m.text))
+                self.log.debug(u'Invalid distance: ' + repr(m.text))
         elif m.header == u'timerstat':
             self.timerstat = m.text
         elif m.header == u'bunches':
@@ -489,20 +493,18 @@ class rr_announce(object):
                 # and advance one second
                 self.nc += tod.ONE
             else:
-                print(u'Timeout called early: ' + ntime.rawtime())
-                #self.log.debug(u'Timeout called early: ' + ntime.rawtime())
+                self.log.debug(u'Timeout called early: ' + ntime.rawtime())
                 # no need to advance, desired timeout not yet reached
         except Exception as e:
-            print(u'Timeout: ' + unicode(e))
-            #self.log.error(u'Timeout: ' + unicode(e))
+            self.log.error(u'Timeout: ' + unicode(e))
 
         # 3: Re-Schedule
         tt = tod.tod(u'now')+tod.tod(u'0.01')
         while self.nc < tt:     # ensure interval is positive
             if tod.MAX - tt < tod.ONE:
-                #self.log.debug(u'Midnight rollover.')
+                self.log.debug(u'Midnight rollover.')
                 break
-            #self.log.debug(u'May have missed an interval, catching up.')
+            self.log.debug(u'May have missed an interval, catching up.')
             self.nc += tod.ONE  # 0.01 allows for processing delay
         ival = int(1000.0 * float((self.nc - tod.tod(u'now')).timeval))
         glib.timeout_add(ival, self.timeout)
@@ -512,9 +514,9 @@ class rr_announce(object):
 
     def process_timeout(self):
         """Perform required timeout activities."""
+        estr = u''
+        dstr = u''
         if self.elapstart is not None:
-            estr = u''
-            dstr = u''
             if self.elapfin is not None:
                 # Race over - show elap and down if poss
                 estr = (self.elapfin - self.elapstart).rawtime(0)
@@ -523,8 +525,10 @@ class rr_announce(object):
                         dstr = u'+' + (self.tod - self.elapfin).rawtime(0)
             else:
                 # race in progress, show run time and distance or lap
-                estr = (self.tod -
-                        (self.elapstart+tod.tod('0.5'))).rawtime(0)
+                elap = self.tod - (self.elapstart+tod.tod('0.2'))
+                if elap > tod.MAXELAP:
+                    elap = tod.ZERO
+                estr = elap.rawtime(0)
                 if self.distance is not None and self.distance > 0.5:
                     dstr = u'~{0:1.1f}km'.format(self.distance)
                 if self.lapfin is not None:
@@ -532,15 +536,21 @@ class rr_announce(object):
                     laptm = self.tod - self.lapfin
                     if laptm < self.maxlaptime:
                         dstr = u'+' + laptm.rawtime(0)
-            self.elap_lbl.set_text(estr)
-            self.gap_lbl.set_text(dstr)
+        self.elap_lbl.set_text(estr)
+        self.gap_lbl.set_text(dstr)
 
         if self.timerstat == 'running' and self.laplbl:
-            # overwrite title with lap and sprint info
-            tmsg = self.laplbl
+            tmsg = u''
+            if self.finstr:
+                tmsg = self.finstr + u' '
+            # add lap and sprint info
+            tmsg += self.laplbl
             if self.laptype:
                 tmsg += u' - ' + self.laptype
             self.lbl_header.set_text(tmsg)
+        else:
+            if self.title_str:
+                self.lbl_header.set_text(self.title_str)
 
         if self.redraw_flag:
             self.redraw_flag = False
@@ -602,7 +612,7 @@ class rr_announce(object):
         self.elap_lbl.set_text('-h--:--')
         self.elap_lbl.modify_font(pango.FontDescription(FONTFACE + ' '+fnszstr))
         self.gap_lbl = b.get_object('gap_lbl')
-        self.gap_lbl.set_text('+--:--')
+        self.gap_lbl.set_text('~jjj.lkm')
         self.gap_lbl.modify_font(pango.FontDescription(FONTFACE + ' '+fnszstr))
         self.map_winsz = 0
         self.map_xoft = 0
@@ -651,10 +661,11 @@ class rr_announce(object):
         b.connect_signals(self)
 
         # animation variables
+        self.title_str = None
         self.final = False
         self.elapstart = None
         self.elapfin = None
-        self.finstr = None
+        self.finstr = None	# used with lapstr
         self.timerstat = u'running'     # default assume run state
         self.distance = None
         self.laplbl = None
@@ -666,7 +677,7 @@ class rr_announce(object):
         self.tod = tod.tod(u'now').truncate(0)
         self.nc = self.tod + tod.tod(u'1.22') # set interval a little off mark
         self.maxlaptime = tod.tod('2:00') # default maximum lap time
-        #self.log.debug(u'Starting clock intervals at: ' + self.nc.rawtime(3))
+        self.log.debug(u'Starting clock intervals at: ' + self.nc.rawtime(3))
         glib.timeout_add(2000, self.timeout)
 
 
