@@ -55,19 +55,25 @@ import time		# for time.strftime() :(
 import metarace
 from metarace import strops
 from metarace import unt4
-from metarace import sender
 from metarace import tod
 
 PAGE_INIT=10		# count before table starts displaying
-PAGE_DELAY=70		# def tenths of sec to hold each page of table
-PAGE_ROWOFT=11		# first DHI row for table data
-SP_PAGE_ROWOFT=6	# first DHI row for table data
+PAGE_DELAY=50		# def tenths of sec to hold each page of table
+PAGE_ROWOFT=0		# first DHI row for table data
+SP_PAGE_ROWOFT=0	# first DHI row for table data
 DATE_FMT=('%a %d/%m/%y')
 
-def get_dateline():
-    return (time.strftime(DATE_FMT)
-            + u' ' * (metarace.SCB_LINELEN - 22)
-            + tod.tod('now').meridian())
+def get_dateline(width=32):
+    dpart = time.strftime(DATE_FMT) 
+    tpart = tod.tod('now').meridian()
+    ret = tpart 
+    totlen = len(tpart)+len(dpart)
+    if totlen >= width: # with a space
+        ret = tpart.center(width)	# fall back to time
+    else:
+        ret = dpart + u' ' * (width-totlen) + tpart
+
+    return ret
 
 class scbwin(object):
     """Base class for all scoreboard windows.
@@ -127,7 +133,10 @@ class scbclock(scbwin):
         self.line1 = line1
         self.line2 = line2
         self.line3 = line3
-        self.header = get_dateline()
+        self.bodyoft = 1 # body text offset
+        if self.scb.pagelen > 4:
+            self.bodyoft = 2
+        self.header = get_dateline(self.scb.linelen)
         self.tovr = unt4.OVERLAY_4LINE
         if self.line3 == u'':
             if self.line2 == u'':
@@ -138,10 +147,11 @@ class scbclock(scbwin):
             else:
                 self.tovr = unt4.OVERLAY_3LINE
     def redraw(self):
-        for i in range(0,4):
+        for i in range(0,self.scb.pagelen):
             self.scb.clrline(i)
-        self.scb.setoverlay(self.tovr)
+        #self.scb.setoverlay(self.tovr)
         self.scb.setline(0, self.header)
+        #self.scb.flush()
 
     def update(self):
         """Animate the clock window.
@@ -154,18 +164,28 @@ class scbclock(scbwin):
 
         """
         if not self.paused:
+            flush = False
             if self.count == 14:
-                self.scb.setline(1,self.line1)
+                self.scb.setline(self.bodyoft,
+                                 self.line1.strip().center(self.scb.linelen))
+                flush = True
             if self.count == 16:
-                self.scb.setline(2,self.line2)
+                self.scb.setline(self.bodyoft+1,
+                                 self.line2.strip().center(self.scb.linelen))
+                flush = True
             if self.count == 18:
-                self.scb.setline(3,self.line3)
+                self.scb.setline(self.bodyoft+2,
+                                 self.line3.strip().center(self.scb.linelen))
+                flush = True
             if self.count % 2 == 0:
-                next = get_dateline()
+                next = get_dateline(self.scb.linelen)
                 if next != self.header:
                     self.scb.setline(0, next)
+                    flush = True
                     self.header = next
             self.count += 1
+            if flush:
+                self.scb.flush()
 
 class scbtt(scbwin):
     """Pursuit/ITT/Teams Timer window.
@@ -200,14 +220,18 @@ class scbtt(scbwin):
         self.nextt2 = u''
         self.curr2 = u''
         self.nextr2 = u''
+        self.singleoft = 0
+
+    def set_single(self):
+        self.singleoft = 1
 
     def redraw(self):
-        for i in range(4,10):
+        for i in range(0,10):
             self.scb.clrline(i)
-        self.scb.setoverlay(unt4.OVERLAY_R1P4)
-        self.scb.setline(4, self.header)
-        self.scb.setline(6, self.line1)
-        self.scb.setline(8, self.line2)
+        #self.scb.setoverlay(unt4.OVERLAY_R1P4)
+        self.scb.setline(0, self.header)
+        self.scb.setline(1+self.singleoft, self.line1)
+        self.scb.setline(3, self.line2)
         self.nextline2 = self.line2
         self.curt1 = u''
         self.nextt1 = u''
@@ -217,6 +241,7 @@ class scbtt(scbwin):
         self.nextt2 = u''
         self.curr2 = u''
         self.nextr2 = u''
+        self.scb.flush()
 
     def setline2(self, line2str=u''):
         """Replace the line 2 text."""
@@ -241,29 +266,29 @@ class scbtt(scbwin):
     def update(self):
         """If any time or ranks change, copy new value onto overlay."""
         if not self.paused:
-            if self.curr1 != self.nextr1:
-                self.scb.setline(7,
-                      strops.truncpad(self.nextr1, metarace.SCB_LINELEN - 13,
+            flush = False
+            if self.curr1 != self.nextr1 or self.curt1 != self.nextt1:
+                self.scb.setline(2+self.singleoft,
+                      strops.truncpad(self.nextr1, self.scb.linelen - 13,
                                       'r') + u' ' + self.nextt1)
                 self.curr1 = self.nextr1
                 self.curt1 = self.nextt1
-            elif self.curt1 != self.nextt1:
-                self.scb.postxt(7, metarace.SCB_LINELEN - 12, self.nextt1)
-                self.curt1 = self.nextt1
-            if self.curr2 != self.nextr2:
-                self.scb.setline(9,
-                      strops.truncpad(self.nextr2, metarace.SCB_LINELEN - 13,
+                flush = True
+            if self.curr2 != self.nextr2 or self.curt2 != self.nextt2:
+                self.scb.setline(4,
+                      strops.truncpad(self.nextr2, self.scb.linelen - 13,
                                       'r') + u' ' + self.nextt2)
                 self.curr2 = self.nextr2
                 self.curt2 = self.nextt2
-            elif self.curt2 != self.nextt2:
-                self.scb.postxt(9, metarace.SCB_LINELEN - 12, self.nextt2)
-                self.curt2 = self.nextt2
+                flush = True
             if self.line2 != self.nextline2:
                 # A hack to allow display of speed string at single finish
                 self.line2 = self.nextline2
-                self.scb.setline(8, self.nextline2)
+                self.scb.setline(3, self.nextline2)
+                flush = True
             self.count += 1
+            if flush:
+                self.scb.flush()
 
 class scbtimer(scbwin):
     """Sprint timer window with avg speed.
@@ -295,17 +320,18 @@ class scbtimer(scbwin):
         self.nextavg = u''
 
     def redraw(self):
-        for i in range(4,10):
+        for i in range(0,10):
             self.scb.clrline(i)
-        self.scb.setoverlay(unt4.OVERLAY_R2P2)
-        self.scb.setline(4, self.line1)
-        self.scb.setline(5, self.line2)
-        self.scb.setline(6, strops.truncpad(self.timepfx, 
-                             metarace.SCB_LINELEN - 13, 'r'))
+        #self.scb.setoverlay(unt4.OVERLAY_R2P2)
+        self.scb.setline(0, self.line1.strip().center(self.scb.linelen))
+        self.scb.setline(1, self.line2.strip().center(self.scb.linelen))
+        self.scb.setline(3, strops.truncpad(self.timepfx, 
+                             self.scb.linelen - 13, 'r'))
         self.curtime = u''
         self.nexttime = u''
         self.curavg = u''
         self.nextavg = u''
+        self.scb.flush()
 
     def settime(self, timestr=u''):
         """Set the next time speed string."""
@@ -318,16 +344,21 @@ class scbtimer(scbwin):
     def update(self):
         """If time or avg change, copy new value onto overlay."""
         if not self.paused:
+            flush = False
             if self.curtime != self.nexttime:
-                self.scb.postxt(6, metarace.SCB_LINELEN - 12,
+                self.scb.postxt(3, self.scb.linelen - 12,
                                   self.nexttime)
                 self.curtime = self.nexttime
+                flush = True
             if self.curavg != self.nextavg:
-                self.scb.setline(7,
-                      strops.truncpad(self.avgpfx, metarace.SCB_LINELEN - 13,
+                self.scb.setline(4,
+                      strops.truncpad(self.avgpfx, self.scb.linelen - 13,
                                        'r') + u' ' + self.nextavg)
                 self.curavg = self.nextavg
+                flush = True
             self.count += 1
+            if flush:
+                self.scb.flush()
 
 # A rider intro screen
 #
@@ -336,6 +367,8 @@ class scbtimer(scbwin):
 # Line 3: Name/info types out char by char after 2-10th delay
 # Line 4: (optional) types out after further 2-10th delay
 #
+## BROKEN AND UNUSED !!
+#
 # Pauses at completion of draw
 class scbintro(scbwin):
     """Rider intro screen.
@@ -347,25 +380,32 @@ class scbintro(scbwin):
     
     """
     def redraw(self):
-        for i in range(0,4):
+        for i in range(0,10):
             self.scb.clrline(i)
-        self.scb.setoverlay(self.tovr)
+        #self.scb.setoverlay(self.tovr)
         self.scb.setline(0, self.header)
+        self.scb.flush()
 
     def update(self):
         if not self.paused:
+            flush = False
             msgstart = PAGE_INIT + 6
             msgend = msgstart + 25
             if self.count == PAGE_INIT:	# draw prompt at ~+0.5s
-                self.scb.setline(1,self.prompt)
+                self.scb.setline(2,self.prompt)
+                flush = True
             elif self.count >= msgstart and self.count < msgend:
                 oft = self.count-msgstart
                 if len(self.info) > oft:
-                    self.scb.postxt(2, oft, self.info[oft])
+                    self.scb.postxt(3, oft, self.info[oft])
+                    flush = True
             elif self.count == msgend:
-                self.scb.setline(2, self.info)
+                self.scb.setline(3, self.info)
+                flush = True
                 self.paused = True
             self.count += 1
+            if flush:
+                self.scb.flush()
 
     def setinfo(self, prompt=u'', info=u'', xtra=u''):
         """Update overlay info."""
@@ -373,20 +413,21 @@ class scbintro(scbwin):
         self.info = info[0:32]
         self.xtra = xtra[0:32]
 
-    def __init__(self, scb=None, head=u'', lines=3):
+    def __init__(self, scb=None, head=u'', lines=5):
         scbwin.__init__(self, scb)
         self.header = head
-        self.lines = 3
+        self.lines = 5
         self.prompt = u''
         self.info = u''
         self.xtra = u''
         self.tovr = unt4.OVERLAY_3LINE
-        if lines != 3:
-            self.lines = 4
+        if lines != 5:
+            self.lines = 7
             self.tovr = unt4.OVERLAY_4LINE
 
 class logoanim(scbwin):
     """Animated 'logo' screen."""
+    ## NOTE: No effect on text board
     def redraw(self):
         pass
 
@@ -394,7 +435,7 @@ class logoanim(scbwin):
         if not self.paused:
             if self.count % self.delay == 0:
                  dbline = 18
-                 overlay = unt4.OVERLAY_IMAGE
+                 #overlay = unt4.OVERLAY_IMAGE
                  # Alternate overlay
                  if self.curpg == 0:
                      self.curpg = 1
@@ -411,7 +452,7 @@ class logoanim(scbwin):
                      self.scb.setline(dbline, self.llist[curidx]) # set img
                  
                  # select overlay
-                 self.scb.setoverlay(overlay)
+                 #self.scb.setoverlay(overlay)
                  if len(self.llist) == 1:
                      self.paused = True
             self.count += 1
@@ -433,17 +474,18 @@ class logoanim(scbwin):
 class scbtest(scbwin):
     """A "test pattern" that loops over all the overlays."""
     def redraw(self):
-        for i in range(0,19):
+        for i in range(0,10):
             self.scb.setline(i, u'Line ' + unicode(i)) 
+        self.scb.flush()
 
     def update(self):
         if not self.paused:
             if self.count % 40 == 0:
-                 self.scb.setoverlay(self.ovrs[self.curov])
+                 #self.scb.setoverlay(self.ovrs[self.curov])
                  self.curov = (self.curov + 1) % 4
             self.count += 1
 
-    def __init__(self, scb=None, head=u'', lines=3):
+    def __init__(self, scb=None, head=u'', lines=5):
         scbwin.__init__(self, scb)
         self.ovrs = [unt4.OVERLAY_4LINE, unt4.OVERLAY_R2P4, unt4.OVERLAY_24X6,
                      unt4.OVERLAY_IMAGE]
@@ -481,20 +523,23 @@ class scbintsprint(scbwin):
             self.rows.insert(len(self.rows) - 2, u' ')
 
     def redraw(self):
-        for i in range(4,10):
+        for i in range(0,10):
             self.scb.clrline(i)
-        self.scb.setoverlay(self.tovr)
-        self.scb.setline(4, self.line1)
-        self.scb.setline(5, self.line2)
+        #self.scb.setoverlay(self.tovr)
+        #self.scb.setline(0, self.line1)
+        self.scb.setline(0, self.line2)	# for NZL -> roll line 2 up
+        self.scb.flush()
 
     def update(self):
         if self.count%2 == 0 and self.count > PAGE_INIT: # wait ~1/2 sec
             lclk = (self.count - PAGE_INIT) // 2
 	    cpage = (lclk//self.delay) % self.nrpages
             pclk = lclk%self.delay
+            flush = False
             if pclk < self.pagesz + 1:
                 if pclk != self.pagesz:
                     self.scb.clrline(SP_PAGE_ROWOFT + pclk)
+                    flush = True
                 elif self.nrpages == 1:
                     self.count += 1
                     self.paused = True       # no animate on single page
@@ -503,6 +548,9 @@ class scbintsprint(scbwin):
                     if roft < len(self.rows):
                         self.scb.setline(SP_PAGE_ROWOFT + pclk-1,
                                          self.rows[roft])
+                        flush = True
+            if flush:
+                self.scb.flush()
         if not self.paused:
             self.count += 1
 
@@ -576,23 +624,26 @@ class scbtable(scbwin):
                 self.rows.append(u' ')
                 padrows -= 1
             self.rows.append(strops.truncpad(self.timepfx,
-                                  metarace.SCB_LINELEN - 13, 'r')
+                                  self.scb.linelen - 13, 'r')
                                       + u' ' + self.timestr[0:12])
 
     def redraw(self):
-        for i in range(10,18):
+        for i in range(0,10):
             self.scb.clrline(i)
-        self.scb.setoverlay(self.tovr)
-        self.scb.setline(10, self.header)
+        #self.scb.setoverlay(self.tovr)
+        self.scb.setline(0, self.header)	# ? use header?
+        self.scb.flush()
 
     def update(self):
         if self.count%2 == 0 and self.count > PAGE_INIT: # wait ~1/2 sec
             lclk = (self.count - PAGE_INIT) // 2
 	    cpage = (lclk//self.delay) % self.nrpages
             pclk = lclk%self.delay
+            flush = False
             if pclk < self.pagesz + 1:
                 if pclk != self.pagesz:
                     self.scb.clrline(PAGE_ROWOFT + pclk)
+                    flush = True
                 elif self.nrpages == 1:
                     self.count += 1
                     self.paused = True       # no animate on single page
@@ -601,26 +652,30 @@ class scbtable(scbwin):
                     if roft < len(self.rows):
                         self.scb.setline(PAGE_ROWOFT + pclk-1,
                                          self.rows[roft])
+                        flush = True
+            if flush:
+                self.scb.flush()
         if not self.paused:
             self.count += 1
 
     def __init__(self, scb=None, head=u'',
-                 coldesc=None, rows=None, pagesz=7,
+                 coldesc=None, rows=None, pagesz=4,
                  timepfx=u'', timestr=None, delay=PAGE_DELAY):
         scbwin.__init__(self, scb)
         # set page size
-        self.pagesz = 7
+        self.pagesz = 4
         self.nrpages = 0
         self.delay = delay
         self.timestr = timestr
         self.timepfx = timepfx
         self.tovr = unt4.OVERLAY_T1P5
-        if pagesz == 5:
-            self.pagesz = 5		# ignore any other size
+        if pagesz == 4:
+            self.pagesz = 4		# ignore any other size
             self.tovr = unt4.OVERLAY_T1P4
 
         # prepare header -> must be preformatted
-        self.header = head[0:metarace.SCB_LINELEN]
+        ## TODO : selective pagesz with offset based on header
+        self.header = head[0:self.scb.linelen]
 
         # load rows
         self.rows = []		# formatted rows
