@@ -31,13 +31,19 @@ import threading
 import Queue
 import logging
 import socket
+import metarace
 
 from metarace import unt4
 from metarace import strops
 
 # Default Galactica encoding is cp1252 (ansi)
-#ENCODING = 'ascii'
 ENCODING = 'cp1252'
+LINELEN = 32
+PAGELEN = 8
+
+# protocol to object mappings
+PROTOCOLS = {
+}
 
 # dispatch thread queue commands
 TCMDS = ('EXIT', 'PORT', 'MSG')
@@ -54,13 +60,15 @@ class scbport(object):
 
         """
         self.__s = socket.socket(socket.AF_INET, protocol)
-        self.__s.connect(addr)
         if protocol is socket.SOCK_STREAM:
             # set the TCP 'no delay' option
             self.__s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         else:	# assume Datagram (UDP)
             # set all scb packets to look like 'EF' VoIP packets
-            self.__s.setsockopt(socket.IPPROTO_IP, socket.IP_TOS, 0xB8)
+            #self.__s.setsockopt(socket.IPPROTO_IP, socket.IP_TOS, 0xB8)
+            # enable broadcast send
+            self.__s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
+        self.__s.connect(addr)
         self.send = self.__s.send  # local cache the send() method
         self.running = True
 
@@ -83,10 +91,10 @@ class scbport(object):
         except:
             pass	# error here should not leak out
 
-def mkport(port):
+def mkport(port=None):
     """Create a new scbport socket object.
 
-    port is a string specifying the DHI address as follows:
+    port is a string specifying the address as follows:
 
         [PROTOCOL:]ADDRESS[:PORT]
 
@@ -96,17 +104,17 @@ def mkport(port):
         ADDRESS :: hostname or IP address
         PORT :: port name or number (optional)
 
-    The default protocol is UDP and the default port 5060.
-
     """
     nprot = socket.SOCK_DGRAM	# default is UDP
     naddr = u'localhost'	# default is localhost
     nport = 5060		# default is 'sip' scbport
-    if port == u'DISC':
-        nprot = socket.SOCK_STREAM
-        naddr = u'192.168.1.212'	# specific to DISC velodrome
-        nport = 16372
-    elif port == u'DEBUG':
+
+    # import system defaults if required
+    if metarace.sysconf.has_option(u'sender',u'portspec'):
+        if not port or port == u'DEFAULT':
+            port = metarace.sysconf.get(u'sender',u'portspec')
+    
+    if port == u'DEBUG': # force use of the hardcoded UDP endpoint
         pass
     else:
         vels = [u'UDP', u'localhost', u'sip']
@@ -131,11 +139,14 @@ def mkport(port):
         # 'import' the vels...
         if vels[0] == u'TCP':
             nprot = socket.SOCK_STREAM
+            nport = 16372
         elif vels[0] == u'UDP':
             nprot = socket.SOCK_DGRAM
+            nport = 5060
         else:
             raise socket.error('Invalid protocol specified.')
         naddr = vels[1]
+        # override port if supplied
         if vels[2].isdigit():
             nport = int(vels[2])
         else:
@@ -144,6 +155,29 @@ def mkport(port):
     ## split port string into [PROTOCOL:]ADDR[:PORT]
     return scbport((naddr, nport), nprot)
 
+def mksender(port=None, protocol=None):
+    nprot = u'dhi'
+    if metarace.sysconf.has_option(u'sender', u'protocol'):
+        look = metarace.sysconf.get(u'sender', u'protocol').lower()
+        if look in PROTOCOLS:
+            nprot = look
+    if protocol is not None and protocol in PROTOCOLS:
+        nprot = protocol
+    cols = LINELEN
+    rows = PAGELEN
+    encoding = ENCODING
+    if metarace.sysconf.has_option(u'sender', u'linelen'):
+        cols = metarace.sysconf.get(u'sender', u'linelen')
+    if metarace.sysconf.has_option(u'sender', u'pagelen'):
+        rows = metarace.sysconf.get(u'sender', u'pagelen')
+    if metarace.sysconf.has_option(u'sender', u'encoding'):
+        encoding = metarace.sysconf.get(u'sender', u'encoding')
+    ret = PROTOCOLS[nprot](port)
+    ret.linelen = cols
+    ret.pagelen = rows
+    ret.encoding = encoding
+    return ret
+
 class sender(threading.Thread):
     """Galactica DHI sender thread.
 
@@ -172,6 +206,10 @@ class sender(threading.Thread):
         msg = msg[0:self.linelen].ljust(self.linelen)
         self.sendmsg(unt4.unt4(xx=0,yy=int(line),text=msg))
 
+    def flush(self):
+        """Send a flush packet if possible."""
+        pass
+
     def linefill(self, line, char=u'_'):
         """Use char to fill the specified line."""
         msg = char * self.linelen
@@ -190,12 +228,14 @@ class sender(threading.Thread):
             self.sendmsg(newov)
             self.curov = newov
 
-    def __init__(self, port=None, linelen=32):
+    def __init__(self, port=None):
         """Constructor."""
         threading.Thread.__init__(self) 
         self.name = u'sender'
         self.port = None
-        self.linelen = int(linelen)
+        self.linelen = LINELEN
+        self.pagelen = PAGELEN
+        self.encoding = ENCODING	# set by mksender
         self.ignore = False
         self.curov = None
         self.queue = Queue.Queue()
@@ -265,7 +305,7 @@ class sender(threading.Thread):
             self.queue.task_done()
             try:
                 if m[0] == 'MSG' and not self.ignore and self.port:
-                    self.port.sendall(m[1].encode(ENCODING,'replace'))
+                    self.port.sendall(m[1].encode(self.encoding,'replace'))
                 elif m[0] == 'EXIT':
                     self.log.debug(u'Request to close : ' + str(m[1]))
                     self.running = False
@@ -291,6 +331,50 @@ class sender(threading.Thread):
             self.port.close()
         self.log.info(u'Exiting')
 
+class altsender(sender):
+    def sendmsg(self, unt4msg=None):
+        """Pack and send a unt4 message to the altscb."""
+        self.queue.put_nowait(('MSG', unt4msg.altpack()))
+
+    def setoverlay(self, newov):
+        pass
+
+    def flush(self):
+        """Send a flush packet."""
+        self.queue.put_nowait(('MSG', unt4.ALT_ACK.altpack()+(9 * unichr(unt4.NUL))))
+
+    def linefill(self, line, char=u'_'):
+        """Use char to fill the specified line."""
+        msg = char * self.linelen
+        umsg = unt4.unt4(xx=0,yy=int(line),text=msg)
+        self.sendmsg(umsg)
+        self.sendmsg(umsg)
+
+    def postxt(self, line, oft, msg):
+        """Position msg at oft on line in DHI database."""
+        # TODO: Handle negative offsets? From right?
+        if oft < self.linelen:
+            msg = msg[0:(self.linelen-oft)]
+            umsg = unt4.unt4(xx=int(oft),yy=int(line),text=msg)
+            self.sendmsg(umsg)
+            self.sendmsg(umsg)
+
+    def clrline(self, line):
+        """Clear the specified line in DHI database."""
+        umsg = unt4.unt4(xx=0,yy=int(line),text=u' '*self.linelen)
+        self.sendmsg(umsg)
+        self.sendmsg(umsg)
+
+    def setline(self, line, msg):
+        """Set the specified DHI database line to msg."""
+        msg = msg[0:self.linelen].ljust(self.linelen)
+        umsg = unt4.unt4(xx=0,yy=int(line),text=msg)
+        self.sendmsg(umsg)
+        self.sendmsg(umsg)
+
+PROTOCOLS[u'dhi']=sender
+PROTOCOLS[u'ivc']=altsender
+
 if __name__ == "__main__":
     """Simple 'Hello World' example with logging."""
     h = logging.StreamHandler()
