@@ -15,6 +15,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+## TODO : Rename and fix for use as a general purpose graphical display,
+##        requires paging support and some sort of transition
+
 """Text-only scoreboard for VGA videowall (Launceston)
 
 This module implements a simple scoreboard that mimics the Omega
@@ -364,7 +367,7 @@ def main():
     app.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
         app.shutdown()
         raise
