@@ -56,9 +56,8 @@ from metarace import printing
 
 LOGFILE = u'event.log'
 LOGHANDLER_LEVEL = logging.DEBUG
-DEFANNOUNCE_PORT = u''
 CONFIGFILE = u'config.json'
-TRACKMEET_ID = u'trackmeet_1.7'	# configuration versioning
+TRACKMEET_ID = u'trackmeet_1.8'	# configuration versioning
 EXPORTPATH = u'export'
 
 def mkrace(meet, event, ui=True):
@@ -199,6 +198,7 @@ class trackmeet:
             self.tracklen_d = tld.get_value_as_int()
             nport = spe.get_text().decode('utf-8','replace')
             if nport != self.scbport:
+                # TODO: swap type handler if necessary
                 self.scbport = nport
                 self.scb.setport(nport)
             nport = upe.get_text().decode('utf-8','replace')
@@ -293,11 +293,14 @@ class trackmeet:
         print_op.set_default_page_setup(self.pageset)
         print_op.connect("begin_print", self.begin_print, rep)
         print_op.connect("draw_page", self.draw_print_page, rep)
+        self.log.info('calling into print op...')
         res = print_op.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG,
                                self.window)
+        self.log.info('print op returned: ' + repr(res))
         if res == gtk.PRINT_OPERATION_RESULT_APPLY:
             self.printprefs = print_op.get_print_settings()
             self.log.debug(u'Updated print preferences.')
+        self.log.info('returning false from print_report')
         return False
 
     def begin_print(self,  operation, context, rep):
@@ -320,6 +323,35 @@ class trackmeet:
             self.printprefs = dlg.get_print_settings()
             self.log.info(u'Updated print preferences.')
 
+    ## Event action callbacks
+    def eventdb_cb(self, evlist, reptype=None):
+        """Make a report containing start lists for the events listed."""
+        secs = []
+        reptypestr = reptype.title()
+        for eno in evlist:
+            e = self.edb[eno]
+            h = mkrace(self, e, False)
+            h.loadconfig()
+            if reptype == 'startlist':
+                secs.extend(h.startlist_report())
+            elif reptype == 'result':
+                reptypestr = 'Results'
+                secs.extend(h.result_report())
+            elif reptype == 'program':
+                reptypestr = 'Program of Events'
+                secs.extend(h.startlist_report(True))	# startlist in program
+            else:
+                self.log.error(u'Unknown report type in eventdb calback: '
+                                 + repr(reptype))
+            h.destroy()
+        if len(secs) > 0:
+            self.log.info(reptype + u' callback: ' + u', '.join(evlist))
+            self.print_report(secs, doprint=false, docstr=reptypestr,
+                                    exportfile=u'trackmeet_' + reptype)
+        else:
+            self.log.info(u'Startlist callback: Nothing to report.')
+        return False
+
     ## Race menu callbacks.
     def menu_race_startlist_activate_cb(self, menuitem, data=None):
         """Generate a startlist."""
@@ -565,7 +597,6 @@ class trackmeet:
                           exportfile=filebase.translate(strops.WEBFILE_UTRANS))
 
     def printprogram(self, prov=True):
-        # TODO:: Jnrs, just map in xtra infos
         tmap = {}
         rlist = {}
         rmap = {}
@@ -606,156 +637,103 @@ class trackmeet:
 
         r.set_provisional(prov)
 
-## !!! NO this is already in the event db !!!
-        # load in extra event meta from eventsrc.csv
-        evmap = {}
-        emfile = os.path.join(self.configpath, 'eventsrc.csv')
-        if os.path.isfile(emfile):
-            self.log.info('Reading extended race info from ' + repr(emfile))
-            with open(emfile, 'rb') as f:
-                cr = csv.reader(f)
-                for ri in cr:
-                    evno = None
-                    evname = None
-                    xtra1 = ''
-                    xtra2 = ''
-                    lenstr = ''
-                    timestr = None
-                    holders = 0
-                    showcats = False
-                    phase = None
-                    record = None
-                    heatlist = []
-                    if len(ri) > 2:
-                        evno = ri[0]
-                        evname = ri[1] + ' ' + ri[2]
-                    if evno and evno != 'Num':
-                        head = 'Event ' + evno + ' ' + evname
-                        if len(ri) > 4 and ri[4]:
-                            etype = ri[4]
-                            if etype in ['keirin', 'sprint']:
-                                timestr = '200m:'
-                            elif etype == 'omnium':
-                                timestr = ''
-                            else:
-                                timestr = 'Time:'
-                        if len(ri) > 9 and ri[9]:
-                            lenstr = ri[9] + ' Lap'
-                            try:
-                                if int(float(ri[9])) > 1:
-                                    lenstr += 's'
-                            except:
-                                pass	# really should check numeric err
-                        if len(ri) > 10 and ri[10]:
-                            xtra1 = ri[10]
-                        if len(ri) > 11 and ri[11]:
-                            xtra2 = ri[11]
-                        if len(ri) > 12 and ri[12]:
-                            holders = int(ri[12])
-                        if len(ri) > 13 and ri[13]:
-                            showcats = strops.confopt_bool(ri[13])
-                        if len(ri) > 14 and ri[14]:
-                            phase = ri[14]
-                        if len(ri) > 15 and ri[15]:
-                            record = ri[15]
-                        if len(ri) > 16 and ri[16]:
-                            heatlist = strops.listsplit(ri[16])
-                        evmap[evno] = [head, lenstr, xtra1, xtra2,
-                                       timestr, holders, showcats,
-                                       phase, record, heatlist]
-
         for e in self.edb:
             h = mkrace(self, e, False)
             h.loadconfig()
-            sec = None
-            gapcount = 0
-            showcats = False
-            if h.evtype in ['classification', 'break']:
-                h.destroy()
-                continue	# skip events that can't be in program?
-            if h.evtype in ['flying 200', 'flying lap',
-                                     'indiv tt', 'indiv pursuit',
-                                     'pursuit race', 'team pursuit',
-                                      'team pursuit race']:
-                sec = printing.dual_ittt_startlist()
-                if h.evno in evmap:
-                    if 'race' not in h.evtype:
-                        sec.showheats = True
-                    rmeta = evmap[h.evno]
-                    sec.heading = rmeta[0]
-                    sec.subheading = '\t'.join([rmeta[1], rmeta[3],
-                                            rmeta[2]]).strip()
-                    if rmeta[5] > 0:	# placeholders - write in gaps
-                        gapcount = rmeta[5]
-                    showcats = rmeta[6]
-                    if rmeta[8]:
-                        sec.set_record(rmeta[8])
-                else:
-                    sec.heading = self.racenamecat(e, 64).strip()
-                if h.evtype in ['flying 200', 'flying lap']:
-                    sec.set_single()
-                sec.lines = h.get_heats(gapcount)
-                # hack in the cats 
-                if showcats: # how to show cat?
-                    pass
-            elif h.evtype == 'sprint round':	# sudden-death sprint round
-                sec = printing.sprintround()
+            #sec = None
+            #gapcount = 0
+            #showcats = False
+            #if h.evtype in ['classification', 'break']:
+                #h.destroy()
+                #continue	# skip events that can't be in program?
+            #if h.evtype in ['flying 200', 'flying lap',
+                                     #'indiv tt', 'indiv pursuit',
+                                     #'pursuit race', 'team pursuit',
+                                      #'team pursuit race']:
+                #sec = printing.dual_ittt_startlist()
+### ?!?!
+                #if h.evno in evmap:
+                    #if 'race' not in h.evtype:
+                        #sec.showheats = True
+                    #rmeta = evmap[h.evno]
+                    #sec.heading = rmeta[0]
+                    #sec.subheading = '\t'.join([rmeta[1], rmeta[3],
+                                            #rmeta[2]]).strip()
+                    #if rmeta[5] > 0:	# placeholders - write in gaps
+                        #gapcount = rmeta[5]
+                    #showcats = rmeta[6]
+                    #if rmeta[8]:
+                        #sec.set_record(rmeta[8])
+                #else:
+                    #sec.heading = self.racenamecat(e, 64).strip()
+                #if h.evtype in ['flying 200', 'flying lap']:
+                    #sec.set_single()
+                #sec.lines = h.get_heats(gapcount)
+                ## hack in the cats 
+                #if showcats: # how to show cat?
+                    #pass
+            #elif h.evtype == 'sprint round':	# sudden-death sprint round
+                #sec = printing.sprintround()
+                ## TEMP: just use placeholders
+                #if h.evno in evmap:
+                    #rhold = [None, None, None, None, None, None]
+                    #rmeta = evmap[h.evno]
+                    #sec.heading = rmeta[0]
+                    #sec.subheading = '\t'.join([rmeta[1], rmeta[3],
+                                            #rmeta[2]]).strip()
+                    #for heat in rmeta[9]:
+                        #sec.lines.append([heat, rhold, rhold, None])
+            #elif h.evtype == 'sprint final':	# best of 3 sprint final
+                #sec = printing.sprintfinal()
                 # TEMP: just use placeholders
-                if h.evno in evmap:
-                    rhold = [None, None, None, None, None, None]
-                    rmeta = evmap[h.evno]
-                    sec.heading = rmeta[0]
-                    sec.subheading = '\t'.join([rmeta[1], rmeta[3],
-                                            rmeta[2]]).strip()
-                    for heat in rmeta[9]:
-                        sec.lines.append([heat, rhold, rhold, None])
-            elif h.evtype == 'sprint final':	# best of 3 sprint final
-                sec = printing.sprintfinal()
-                # TEMP: just use placeholders
-                if h.evno in evmap:
-                    rhold = [None, None, None, None, None, None]
-                    rmeta = evmap[h.evno]
-                    sec.heading = rmeta[0]
-                    sec.subheading = '\t'.join([rmeta[1], rmeta[3],
-                                            rmeta[2]]).split()
-                    for heat in rmeta[9]:
-                        sec.lines.append([heat, rhold, rhold, None])
-            else:
-                sec = printing.twocol_startlist()
-                if h.evno in evmap:
-                    rmeta = evmap[h.evno]
-                    showcats = rmeta[6]
+                #if h.evno in evmap:
+                    #rhold = [None, None, None, None, None, None]
+                    #rmeta = evmap[h.evno]
+                    #sec.heading = rmeta[0]
+                    #sec.subheading = '\t'.join([rmeta[1], rmeta[3],
+                                            #rmeta[2]]).split()
+                    #for heat in rmeta[9]:
+                        #sec.lines.append([heat, rhold, rhold, None])
+            #else:
+                #sec = printing.twocol_startlist()
+                #if h.evno in evmap:
+                    #rmeta = evmap[h.evno]
+                    #showcats = rmeta[6]
+#
+                #for res in h.result_gen():
+                    #bib = res[0]
+                    #rank = res[1]
+                    #if type(rank) is int:
+                        #rank = unicode(rank) + u'.'
+                    #name = None
+                    #info = res[3]
+                    #if res[0] in rmap:
+                        #name = rmap[res[0]]
+                        #if showcats and res[0] in catmap:
+                            #name += ' ' + catmap[res[0]]
+                    #sec.lines.append([rank, bib, name, info, None, None])
+                #if h.evno in evmap:
+                    #rmeta = evmap[h.evno]
+                    #sec.heading = rmeta[0]
+                    #sec.lenstr = rmeta[1]
+                    #sec.xtra1 = rmeta[2]
+                    #sec.xtra2 = rmeta[3]
+                    #sec.timestr = rmeta[4]
+#
+                    #if rmeta[5] > 0:	# placeholders - write in gaps
+                        #sec.lines = []
+                        #while len(sec.lines) < rmeta[5]:
+                            #sec.lines.append([None, None, None, None, None, None])
+                #else:
+                    #sec.heading = self.racenamecat(e, 64).strip()
+            #r.add_section(sec)
 
-                for res in h.result_gen():
-                    bib = res[0]
-                    rank = res[1]
-                    if type(rank) is int:
-                        rank = unicode(rank) + u'.'
-                    name = None
-                    info = res[3]
-                    if res[0] in rmap:
-                        name = rmap[res[0]]
-                        if showcats and res[0] in catmap:
-                            name += ' ' + catmap[res[0]]
-                    sec.lines.append([rank, bib, name, info, None, None])
-                if h.evno in evmap:
-                    rmeta = evmap[h.evno]
-                    sec.heading = rmeta[0]
-                    sec.lenstr = rmeta[1]
-                    sec.xtra1 = rmeta[2]
-                    sec.xtra2 = rmeta[3]
-                    sec.timestr = rmeta[4]
-
-                    if rmeta[5] > 0:	# placeholders - write in gaps
-                        sec.lines = []
-                        while len(sec.lines) < rmeta[5]:
-                            sec.lines.append([None, None, None, None, None, None])
-                else:
-                    sec.heading = self.racenamecat(e, 64).strip()
-            r.add_section(sec)
+            s = h.startlist_report(True)
+            for sec in s:
+                r.add_section(sec)
             h.destroy()
                 
-        # Add team abbrevs
+        # Add team abbrevs, if available
         s = printing.twocol_startlist()
         s.heading = 'Abbreviations'
         s.lines = tlist
@@ -1012,13 +990,9 @@ class trackmeet:
                     orep = printing.printrep(template=tfile,
                                              path=self.configpath)
                     orep.strings[u'title'] = self.titlestr
-                    # subtitle should probably be property of meet
                     orep.strings[u'subtitle'] = evstr
                     orep.strings[u'datestr'] = strops.promptstr(u'Date:',
                                                              self.datestr)
-                    #orep.strings['commstr'] = strops.promptstr('Chief Commissaire:',
-                                                  #self.commstr)
-                    #orep.strings['orgstr'] = strops.promptstr('Organiser: ', self.orgstr)
                     orep.strings[u'diststr'] = self.locstr
                     orep.strings[u'docstr'] = evstr
                     if etype in [u'classification']:
@@ -1168,7 +1142,7 @@ class trackmeet:
     ## Menu button callbacks
     def menu_clock_clicked_cb(self, button, data=None):
         """Handle click on menubar clock."""
-        (line1, line2, line3) = strops.titlesplit(self.titlestr + u' ' + self.subtitlestr)
+        (line1, line2, line3) = strops.titlesplit(self.titlestr + u' ' + self.subtitlestr, self.scb.linelen)
         self.scbwin = scbwin.scbclock(self.scb, line1, line2, line3)
         self.scbwin.reset()
         self.log.debug(u'Displaying meet info and clock on scoreboard.')
@@ -1234,19 +1208,21 @@ class trackmeet:
             ret = u' '.join([self.titlestr, self.subtitlestr]).strip()
         return ret
 
-    def racenamecat(self, event, slen=None):
+    def racenamecat(self, event, slen=None, tail=u''):
         """Concatentate race info for display on scoreboard header line."""
         if slen is None:
-            slen = metarace.SCB_LINELEN
+            slen = self.scb.linelen
         evno = u''
         srcev = event[u'evid']
         if self.showevno and event[u'type'] not in [u'break', u'session']:
             evno = u'Ev ' + srcev
         info = event[u'info']
         prefix = event[u'pref']
-        ret = u' '.join([evno, prefix, info]).strip()
+        ret = u' '.join([evno, prefix, info, tail]).strip()
         if len(ret) > slen + 1:
-            ret = u' '.join([evno, info]).strip()
+            ret = u' '.join([evno, info, tail]).strip()
+            if len(ret) > slen + 1:
+                ret = u' '.join([evno, tail]).strip()
         return strops.truncpad(ret, slen)
 
     def racename(self, event):
@@ -1378,40 +1354,40 @@ class trackmeet:
     def saveconfig(self, lastevent=None):
         """Save current meet data to disk."""
         cw = jsonconfig.config()
-        cw.add_section(u'meet')
-        cw.set(u'meet', u'id', TRACKMEET_ID)
+        cw.add_section(u'trackmeet')
+        cw.set(u'trackmeet', u'id', TRACKMEET_ID)
         if self.curevent is not None and self.curevent.winopen:
             self.save_curevent()
-            cw.set(u'meet', u'curevent', self.curevent.evno)
+            cw.set(u'trackmeet', u'curevent', self.curevent.evno)
         elif lastevent is not None:
-            cw.set(u'meet', u'curevent', lastevent)
-        cw.set(u'meet', u'maintimer', self.main_port)
-        cw.set(u'meet', u'backuptimer', self.backup_port)
-        cw.set(u'meet', u'gemini', self.gemport)
-        cw.set(u'meet', u'dbhost', self.dbport)
+            cw.set(u'trackmeet', u'curevent', lastevent)
+        cw.set(u'trackmeet', u'maintimer', self.main_port)
+        cw.set(u'trackmeet', u'backuptimer', self.backup_port)
+        cw.set(u'trackmeet', u'gemini', self.gemport)
+        cw.set(u'trackmeet', u'dbhost', self.dbport)
         if self.timer is self.main_timer:
-            cw.set(u'meet', u'racetimer', u'main')
+            cw.set(u'trackmeet', u'racetimer', u'main')
         else:
-            cw.set(u'meet', u'racetimer', u'backup')
-        cw.set(u'meet', u'scbport', self.scbport)
-        cw.set(u'meet', u'uscbport', self.annport)
-        cw.set(u'meet', u'title', self.titlestr)
-        cw.set(u'meet', u'subtitle', self.subtitlestr)
-        cw.set(u'meet', u'date', self.datestr)
-        cw.set(u'meet', u'location', self.locstr)
-        cw.set(u'meet', u'organiser', self.orgstr)
-        cw.set(u'meet', u'commissaire', self.commstr)
-        cw.set(u'meet', u'logos', self.logos)
-        cw.set(u'meet', u'mirrorpath', self.mirrorpath)
-        cw.set(u'meet', u'mirrorcmd', self.mirrorcmd)
-        cw.set(u'meet', u'clubmode', self.clubmode)
-        cw.set(u'meet', u'showevno', self.showevno)
-        cw.set(u'meet', u'provisional', self.provisional)
-        cw.set(u'meet', u'tracklayout', self.tracklayout)
-        cw.set(u'meet', u'tracklen_n', unicode(self.tracklen_n))  # poss val?
-        cw.set(u'meet', u'tracklen_d', unicode(self.tracklen_d))
-        cw.set(u'meet', u'docindex', unicode(self.docindex))
-        cw.set(u'meet', u'graphicscb', self.graphicscb)
+            cw.set(u'trackmeet', u'racetimer', u'backup')
+        cw.set(u'trackmeet', u'scbport', self.scbport)
+        cw.set(u'trackmeet', u'uscbport', self.annport)
+        cw.set(u'trackmeet', u'title', self.titlestr)
+        cw.set(u'trackmeet', u'subtitle', self.subtitlestr)
+        cw.set(u'trackmeet', u'date', self.datestr)
+        cw.set(u'trackmeet', u'location', self.locstr)
+        cw.set(u'trackmeet', u'organiser', self.orgstr)
+        cw.set(u'trackmeet', u'commissaire', self.commstr)
+        cw.set(u'trackmeet', u'logos', self.logos)
+        cw.set(u'trackmeet', u'mirrorpath', self.mirrorpath)
+        cw.set(u'trackmeet', u'mirrorcmd', self.mirrorcmd)
+        cw.set(u'trackmeet', u'clubmode', self.clubmode)
+        cw.set(u'trackmeet', u'showevno', self.showevno)
+        cw.set(u'trackmeet', u'provisional', self.provisional)
+        cw.set(u'trackmeet', u'tracklayout', self.tracklayout)
+        cw.set(u'trackmeet', u'tracklen_n', unicode(self.tracklen_n))  # poss val?
+        cw.set(u'trackmeet', u'tracklen_d', unicode(self.tracklen_d))
+        cw.set(u'trackmeet', u'docindex', unicode(self.docindex))
+        cw.set(u'trackmeet', u'graphicscb', self.graphicscb)
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
         self.log.debug(u'Saving meet config to ' + repr(cwfilename))
         with open(cwfilename , 'wb') as f:
@@ -1423,16 +1399,16 @@ class trackmeet:
 
     def loadconfig(self):
         """Load meet config from disk."""
-        cr = jsonconfig.config({u'meet':{u'maintimer':timy.MAINPORT,
-                                        u'backuptimer':timy.BACKUPPORT,
+        cr = jsonconfig.config({u'trackmeet':{u'maintimer':u'',
+                                        u'backuptimer':u'',
                                         u'racetimer':u'main',
                                         u'scbport':u'',
-                                        u'uscbport':DEFANNOUNCE_PORT,
+                                        u'uscbport':u'',
                                         u'showevno':True,
 					u'resultnos':True,
                                         u'clubmode':True,
-                                        u'tracklen_n':u'250',
-                                        u'tracklen_d':u'1',
+                                        u'tracklen_n':250,
+                                        u'tracklen_d':1,
                                         u'docindex':u'0',
                                         u'gemini':u'',
                                         u'dbhost':u'',
@@ -1444,13 +1420,14 @@ class trackmeet:
                                         u'commissaire':u'',
                                         u'logos':u'',
                                         u'curevent':u'',
+                                        u'mirrorcmd':u'',
                                         u'mirrorpath':u'',
-                                        u'mirrorcmd':u'echo',
                                         u'graphicscb':u'',
-                                        u'tracklayout':u'DISC',
+                                        u'tracklayout':u'',
                                         u'provisional':False,
                                         u'id':u''}})
-        cr.add_section(u'meet')
+        cr.add_section(u'trackmeet')
+        cr.merge(metarace.sysconf, u'trackmeet')
         cwfilename = os.path.join(self.configpath, CONFIGFILE)
 
         # re-set main log file
@@ -1473,31 +1450,31 @@ class trackmeet:
             self.log.error(u'Reading meet config: ' + unicode(e))
 
         # set main timer port
-        nport = cr.get(u'meet', u'maintimer')
+        nport = cr.get(u'trackmeet', u'maintimer')
         if nport != self.main_port:
             self.main_port = nport
             self.main_timer.setport(nport)
             self.main_timer.sane()
 
         # set backup timer port
-        nport = cr.get(u'meet', u'backuptimer')
+        nport = cr.get(u'trackmeet', u'backuptimer')
         if nport != self.backup_port:
             self.backup_port = nport
             self.backup_timer.setport(nport)
             self.backup_timer.sane()
 
         # add gemini board if defined
-        self.gemport = cr.get(u'meet', u'gemini')
+        self.gemport = cr.get(u'trackmeet', u'gemini')
         if self.gemport != u'':
             self.gemini.setport(self.gemport)
             
         # add database export if present
-        self.dbport = cr.get(u'meet', u'dbhost')
+        self.dbport = cr.get(u'trackmeet', u'dbhost')
         if self.dbport != u'':
             self.db.setport(self.dbport)
 
         # choose race timer
-        if cr.get(u'meet', u'racetimer') == u'main':
+        if cr.get(u'trackmeet', u'racetimer') == u'main':
             if self.timer is self.backup_timer:
                 self.menubut_main.activate()
         else:
@@ -1505,58 +1482,55 @@ class trackmeet:
                 self.menubut_backup.activate()
 
         # choose scoreboard port
-        nport = cr.get(u'meet', u'scbport')
+        nport = cr.get(u'trackmeet', u'scbport')
         if self.scbport != nport:
             self.scbport = nport
             self.scb.setport(nport)
-        self.annport = cr.get(u'meet', u'uscbport')
+        self.annport = cr.get(u'trackmeet', u'uscbport')
         self.announce.set_portstr(self.annport)
         self.announce.clrall()
 
         # set meet meta infos, and then copy into text entries
-        self.titlestr = cr.get(u'meet', u'title')
-        self.subtitlestr = cr.get(u'meet', u'subtitle')
-        self.datestr = cr.get(u'meet', u'date')
-        self.locstr = cr.get(u'meet', u'location')
-        self.orgstr = cr.get(u'meet', u'organiser')
-        self.commstr = cr.get(u'meet', u'commissaire')
-        self.logos = cr.get(u'meet', u'logos')
-        self.mirrorpath = cr.get(u'meet', u'mirrorpath')
-        self.mirrorcmd = cr.get(u'meet', u'mirrorcmd')
+        self.titlestr = cr.get(u'trackmeet', u'title')
+        self.subtitlestr = cr.get(u'trackmeet', u'subtitle')
+        self.datestr = cr.get(u'trackmeet', u'date')
+        self.locstr = cr.get(u'trackmeet', u'location')
+        self.orgstr = cr.get(u'trackmeet', u'organiser')
+        self.commstr = cr.get(u'trackmeet', u'commissaire')
+        self.logos = cr.get(u'trackmeet', u'logos')
+        self.mirrorpath = cr.get(u'trackmeet', u'mirrorpath')
+        self.mirrorcmd = cr.get(u'trackmeet', u'mirrorcmd')
         self.set_title()
 
         # result options (bool)
-        self.clubmode = strops.confopt_bool(cr.get(u'meet', u'clubmode'))
-        self.showevno = strops.confopt_bool(cr.get(u'meet', u'showevno'))
-        self.provisional = strops.confopt_bool(cr.get(u'meet', u'provisional'))
+        self.clubmode = strops.confopt_bool(cr.get(u'trackmeet', u'clubmode'))
+        self.showevno = strops.confopt_bool(cr.get(u'trackmeet', u'showevno'))
+        self.provisional = strops.confopt_bool(cr.get(u'trackmeet', u'provisional'))
 
         # track length
-        n = cr.get(u'meet', u'tracklen_n')
-        d = cr.get(u'meet', u'tracklen_d')
+        n = strops.confopt_posint(cr.get(u'trackmeet', u'tracklen_n'),0)
+        d = strops.confopt_posint(cr.get(u'trackmeet', u'tracklen_d'),0)
         setlen = False
-        if n.isdigit() and d.isdigit():
-            n = int(n)
-            d = int(d)
-            if n > 0 and n < 5500 and d > 0 and d < 10: # sanity check
-                self.tracklen_n = n
-                self.tracklen_d = d
-                setlen = True
+        if n > 0 and n < 5500 and d > 0 and d < 10: # sanity check
+            self.tracklen_n = n
+            self.tracklen_d = d
+            setlen = True
         if not setlen:
             self.log.warn(u'Ignoring invalid track length - default used.')
 
         # track sector lengths
-        self.tracklayout = cr.get(u'meet',u'tracklayout')
+        self.tracklayout = cr.get(u'trackmeet',u'tracklayout')
         self.sectormap = timy.make_sectormap(self.tracklayout)
 
         # document id
-        self.docindex = strops.confopt_posint(cr.get(u'meet', u'docindex'), 0)
+        self.docindex = strops.confopt_posint(cr.get(u'trackmeet', u'docindex'), 0)
 
         self.rdb.clear()
         self.edb.clear()
         self.rdb.load(os.path.join(self.configpath, u'riders.csv'))
         self.edb.load(os.path.join(self.configpath, u'events.csv'))
 
-        cureventno = cr.get(u'meet', u'curevent')
+        cureventno = cr.get(u'trackmeet', u'curevent')
         if cureventno and cureventno in self.edb:
             self.open_event(self.edb[cureventno])
 
@@ -1566,7 +1540,7 @@ class trackmeet:
             try:
                 self.printprefs.load_file(psfilename)
             except Exception as e:
-                self.log.warn(u'Error loading print preferences: ' 
+                self.log.debug(u'Error loading print preferences: ' 
                                  + unicode(e))
 
         # make sure export path exists
@@ -1575,7 +1549,7 @@ class trackmeet:
             self.log.info(u'Created export path: ' + repr(self.exportpath))
 
         # add grpahic scb if required...
-        self.graphicscb = cr.get(u'meet', u'graphicscb')
+        self.graphicscb = cr.get(u'trackmeet', u'graphicscb')
         if self.graphicscb:
             glib.timeout_add_seconds(30, self.gfxscb_connect)
 
@@ -1583,7 +1557,7 @@ class trackmeet:
         # an error message is left on top of status stack. This is not
         # always a hard fail and the user should be left to determine
         # an appropriate outcome.
-        cid = cr.get(u'meet', u'id')
+        cid = cr.get(u'trackmeet', u'id')
         if cid != TRACKMEET_ID:
             self.log.error(u'Meet configuration mismatch: '
                            + repr(cid) + u' != ' + repr(TRACKMEET_ID))
@@ -1670,7 +1644,7 @@ class trackmeet:
         self.pageset.set_right_margin(0, gtk.UNIT_POINTS)
 
         # hardware connections
-        self.scb = sender.sender(u'NULL')
+        self.scb = sender.mksender(u'NULL')
         self.announce = uscbsrv.uscbsrv(80)
         self.scbport = u'NULL'
         self.annport = u''
@@ -1735,8 +1709,11 @@ class trackmeet:
 
         # get event db and pack into scrolled pane
         self.edb = eventdb.eventdb()
+        self.edb.set_startlist_cb(self.eventdb_cb, 'startlist')
+        self.edb.set_result_cb(self.eventdb_cb, 'result')
+        self.edb.set_program_cb(self.eventdb_cb, 'program')
         b.get_object('event_box').add(self.edb.mkview())
-        #self.edb.view.connect('row-activated', self.event_row_activated_cb)
+        ## TODO: event number change handler
         #self.edb.set_evno_change_cb(self.race_evno_change)
 
 	# now, connect each of the race menu types if present in builder
@@ -1766,9 +1743,9 @@ def main():
     app.window.show()
     app.start()
     try:
-        gtk.main()
+        metarace.mainloop()
     except:
-        app.shutdown(u'Exception from gtk.main()')
+        app.shutdown(u'Exception from main loop.')
         raise
 
 if __name__ == '__main__':
