@@ -55,7 +55,7 @@ TAGPREFIX = '058001'		# Wheeltime rfid prefix
 DEFPORT = '192.168.95.32'	# CV wheeltime IP
 WHEELRAWPORT = 10000		# TCP Port for raw tag stream
 WHEELFSPORT = 10200		# TCP Port for FS/LS filtered stream (normal)
-WHEELCMDPORT = 9999		# TCP Wheeltime command port (defunct?)
+WHEELCMDPORT = 8999		# TCP Wheeltime command port (defunct?)
 WHEELPHOTOTHRESH = tod.tod('0.1')	# Wheeltime confidence ~0.1s
 
 # thread queue commands -> private to timy thread
@@ -106,8 +106,12 @@ class wtio(threading.Thread):
         self.log = log
         self.addr = addr
         self.rdbuf = ''
+        self.cb = None
         self.running = False
 
+    def setcb(self, cb=None):
+        self.cb = cb
+
     def close(self):
         """Signal thread for termination."""
         self.running = False
@@ -122,7 +126,7 @@ class wtio(threading.Thread):
         ret = None
         idx = self.rdbuf.find('\n')
         if idx < 0:
-            inb = s.recv(2048)
+            inb = s.recv(512)
             if inb == '':
                 self.log.info('Wheeltime I/O connection broken')
                 self.close()
@@ -148,9 +152,10 @@ class wtio(threading.Thread):
                                    s[30:32], int(s[32:34], 16))
                     istr = '00:0'	# emulate THbC info, but zero for WT
                     chanstr = 'BOX'
-                    self.cqueue.put_nowait(('RFID',
-                                            tod.tod(timestr, index=istr,
-                                                 chan=chanstr, refid=tagid)))
+                    t = tod.tod(timestr, index=istr, chan=chanstr, refid=tagid)
+                    if self.cb is not None:
+                        glib.idle_add(self.cb, t)
+                    self.cqueue.put_nowait(('RFID', t))	# notify only now
                 else:
                     self.log.warn('Spurious tag id: ' + s[4:10] + ' :: ' 
                                     + s[10:16])
@@ -167,9 +172,10 @@ class wtio(threading.Thread):
                 istr = '00:0'
                 chanstr = 'MAN'
                 tagid = ''	# emulate THbC manual trigger (lstripped)
-                self.cqueue.put_nowait(('RFID',
-                                        tod.tod(timestr, index=istr,
-                                                 chan=chanstr, refid=tagid)))
+                t = tod.tod(timestr, index=istr, chan=chanstr, refid=tagid)
+                if self.cb is not None:
+                    glib.idle_add(self.cb, t)
+                self.cqueue.put_nowait(('RFID', t))	# notify only now
             else:
                 self.log.warn('Incorrect char sum message skipped: ' 
                                + hex(sum) + ' != ' + hex(lrc))
@@ -181,7 +187,7 @@ class wtio(threading.Thread):
         self.running = True
         try:
             s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            s.settimeout(10)	# longer timeout is ok now
+            s.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
             s.connect((self.addr, WHEELFSPORT))
             while self.running:
                 try:
@@ -209,13 +215,14 @@ class wheeltime(threading.Thread):
         """
 
         threading.Thread.__init__(self) 
+        self.unitno = None
         self.name = name
         self.addr = None
         self.cqueue = Queue.Queue()	# command queue
         self.log = logging.getLogger(self.name)
         self.log.setLevel(logging.DEBUG)
-        self.setcb()
         self.io = None
+        self.setcb()
         self.running = False
         if port is not None:
             self.setport(port)
@@ -240,6 +247,8 @@ class wheeltime(threading.Thread):
             self.__cb = func
         else:
             self.__cb = self.__defcallback
+        if self.io is not None:
+            self.io.setcb(self.__cb)
 
     def clrmem(self):
         """Clear wheeltime memory."""
@@ -330,11 +339,11 @@ class wheeltime(threading.Thread):
                 # Write phase
                 if m[0] == 'RFID':
                     self.log.log(RFID_LOG_LEVEL, ' ' + str(m[1]))
-                    glib.idle_add(self.__cb, m[1])
+                    #glib.idle_add(self.__cb, m[1])
                 elif m[0] == 'TRIG':
                     if type(m[1]) is tod.tod:
                         self.log.log(RFID_LOG_LEVEL, str(m[1]))
-                        glib.idle_add(self.__cb, m[1])
+                        #glib.idle_add(self.__cb, m[1])
                 elif m[0] == 'MSG':
                     if self.connected():
                         self.command(m[1])
@@ -351,10 +360,12 @@ class wheeltime(threading.Thread):
                         self.io.close()
                         self.io = None
                     if m[1] is not None and m[1] != '' and m[1] != 'NULL':
+                        self.unitno = m[1].split(u'.')[-1]
                         self.addr = m[1]
                         self.log.debug('Re-Connect wheeltime addr: ' + str(m[1]))
                         self.io = wtio(addr=self.addr, cqueue=self.cqueue,
                                        log=self.log)
+                        self.io.setcb(self.__cb)
                         self.io.start()
                     else:
                         self.log.info('Wheeltime not connected.')
@@ -383,7 +394,7 @@ if __name__ == "__main__":
         #w.clrmem()
         #w.sync()
         w.wait()
-        gtk.main()
+        metrarace.mainloop()
     except:
         w.wait()
         w.exit('Exception')
